

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/services/health.proto, blickfeld/diagnostics/services/log.proto, blickfeld/diagnostics/services/report.proto, blickfeld/diagnostics/services/self_test.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Response to health get request."""

    health: "_data__.Health" = betterproto.message_field(1)
    """Aggregated health"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Response to health watch request."""

    health: "_data__.Health" = betterproto.message_field(2)
    """Aggregated health"""


@dataclass(eq=False, repr=False)
class LogStreamRequest(betterproto.Message):
    """Stream request"""

    cursor: "_data__.LogEntryCursor" = betterproto.message_field(1)
    """
    The cursor pointing to the log entry from which to start the stream from.
    This field is optional. If it is not set the stream will start from the
    tail of the journal.   NOTE: To be able to set this field and to specify a
    valid start position in the journal, first, a valid cursor needs to be
    received via the response to this request. Further requests can then stream
    relatively to the received cursor.
    """

    time_direction: "_config__.LogTimeDirection" = betterproto.enum_field(2)
    """
    The streaming direction is used to specify if the service should stream the
    log entries towards older or more recent messages: i.e. backward or forward
    in time. This way, the client can either continuously fetch log messages
    one-by-one from the most recent to the oldest or wait for the occurrence of
    new log entires.
    """


@dataclass(eq=False, repr=False)
class LogStreamResponse(betterproto.Message):
    """Stream response"""

    entry: "_data__.LogEntry" = betterproto.message_field(1)
    """The requested log entry."""


@dataclass(eq=False, repr=False)
class ReportGenerateResponse(betterproto.Message):
    """Stream of the encrypted report in chunks of binary data."""

    binary_chunk: bytes = betterproto.bytes_field(1)
    """Binary chunk of the diagnostics report."""


@dataclass(eq=False, repr=False)
class SelfTestRunRequest(betterproto.Message):
    """The self test run can be configured with this request"""

    self_tests: List["_config__.SelfTest"] = betterproto.enum_field(1)
    """
    Specify self tests which should be executed. If not set, the default set
    will be executed.
    """


@dataclass(eq=False, repr=False)
class SelfTestRunResponse(betterproto.Message):
    """
    The response provides the self test report.   [NOTE] The method will not
    throw an exception on API level if tests failed or have warnings. The
    success flag in the report must be checked.
    """

    report: "_data__.SelfTestReport" = betterproto.message_field(1)
    """Test report"""


class Health(betterproto.ServiceStub):
    """
    This service provides the aggregated health state information of all Qb2
    software modules (only state and state reason are included). Service can be
    used to get an instant overview of the overall Qb2 device health.
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """
        Returns the current aggregated health status of Qb2 software modules.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.diagnostics.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """
        Returns the current aggregated health status of Qb2 software modules.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Returns the aggregated health status stream of Qb2 software modules.
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.diagnostics.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Returns the aggregated health status stream of Qb2 software modules.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Log(betterproto.ServiceStub):
    """
    The Log service provides methods to access the logs of selected software
    modules running on the device. Via this service, the client is able to
    monitor events that happen on the device during operation. This allows to
    asses the state of the device or to have additional information if the
    sensor is in a failed state.
    """

    async def async_stream(
        self,
        *,
        cursor: "_data__.LogEntryCursor" = None,
        time_direction: "_config__.LogTimeDirection" = 0,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["LogStreamResponse"]:
        """
        Is used to request a stream of log entries from the logs of the device.
        """

        request = LogStreamRequest()
        if cursor is not None:
            request.cursor = cursor
        request.time_direction = time_direction

        async for response in self._unary_stream(
            "/blickfeld.diagnostics.services.Log/Stream",
            request,
            LogStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        cursor: "_data__.LogEntryCursor" = None,
        time_direction: "_config__.LogTimeDirection" = 0,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["LogStreamResponse"]:
        """
        Is used to request a stream of log entries from the logs of the device.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            cursor=cursor,
            time_direction=time_direction,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Report(betterproto.ServiceStub):
    """
    The Report service generates an encrypted diagnostics report containing
    information about the state and health information of the Qb2 device.
    """

    async def async_generate(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["ReportGenerateResponse"]:
        """Returns a stream of the report's binary chunks."""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.diagnostics.services.Report/Generate",
            request,
            ReportGenerateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def generate(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["ReportGenerateResponse"]:
        """Returns a stream of the report's binary chunks."""

        loop = asyncio.get_event_loop()
        ait = self.async_generate(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class SelfTest(betterproto.ServiceStub):
    """
    The SelfTest service offers methods to trigger test routines. The routines
    focus on automatic self tests without user interaction. The reports give an
    indication what the potential root cause for an found issue could be.
    """

    async def async_run(
        self,
        *,
        self_tests: Optional[List["_config__.SelfTest"]] = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SelfTestRunResponse":
        """This triggers the default test case routine"""

        self_tests = self_tests or []

        request = SelfTestRunRequest()
        request.self_tests = self_tests

        return await self._unary_unary(
            "/blickfeld.diagnostics.services.SelfTest/Run",
            request,
            SelfTestRunResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def run(
        self,
        *,
        self_tests: Optional[List["_config__.SelfTest"]] = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SelfTestRunResponse":
        """This triggers the default test case routine"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_run(
                self_tests=self_tests,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



import inspect
import logging
import os
import ssl
from types import TracebackType
from typing import Awaitable, Callable, Optional, Type, Union

from grpclib.client import Channel as gRPClibChannel
from grpclib.config import Configuration
from grpclib.events import SendRequest, listen
from grpclib.metadata import _Metadata as Metadata

from .device_ca_cert import ca_cert


class Channel(gRPClibChannel):
    """
    Channel class is used to securely connect to a Qb2 device.

    The connection is encrypted and authenticated.
    The serial_number can be passed additionally to authenticate a particular device.
    If it is not supplied, the authentication only checks if it is a Blickfeld device.

    ::param fqdn_or_ip: Fully qualified domain name e.g. qb2-xxxxx.blickfeld.com, hostname, or IP address
    :type fqdn_or_ip: str
    param serial_number: Unique serial number assigned to each Qb2 device (corresponding serial number is written on the device label)
    :type serial_number: str
    :param port: Port on which the Qb2 device is reachable (the default is 55551)
    :type port: uint
    :param device_ca_cert: String containing a root certificate for SSL connection
    :type device_ca_cert: str
    :param token: String containing an access token or an awaitable which is called on every method call
    """

    def __init__(
        self,
        fqdn_or_ip: str,
        serial_number: Optional[str] = None,
        port: Optional[int] = None,
        device_ca_cert: str = ca_cert,
        token: Optional[Union[str, Callable[["Channel"], str], Awaitable[str]]] = None,
        metadata: Optional[Metadata] = None,
        secure: Optional[bool] = None,
    ) -> None:
        # Store all properties to allow clone
        self.fqdn_or_ip = fqdn_or_ip
        self.serial_number = serial_number
        self.port = port
        self.device_ca_cert = device_ca_cert
        self.token = token
        self.metadata = metadata
        self.secure = (
            os.getenv("BF_ALLOW_INSECURE_CONNECTIONS") is None or token is not None if secure is None else secure
        )

        self.logger = logging.getLogger("Channel")

        # create SSL context for client
        ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)  # client side context
        ctx.verify_mode = ssl.CERT_REQUIRED  # TLS
        ctx.check_hostname = serial_number is not None
        ctx.set_alpn_protocols(["h2"])
        try:
            ctx.load_verify_locations(cadata=device_ca_cert)
        except ssl.SSLError as error:
            raise RuntimeError("Device CA certificate is of incorrect format:" + error.reason)

        # Set name for hostname check to Qb2 device name
        device_name = serial_number + ".qb2.blickfeld.com" if serial_number else None
        config = Configuration(ssl_target_name_override=device_name) if device_name else None

        # Establish TLS connection to verify connectivity
        channel_port = port or 55551
        if not self.secure:
            self.logger.info(f"Using insecure connection for {fqdn_or_ip} as BF_ALLOW_INSECURE_CONNECTIONS is set")
            channel_port = port or 50051
            ctx = None
            config = None

        # establish gRPC secured channel
        super().__init__(host=fqdn_or_ip, port=channel_port, ssl=ctx, config=config)
        # if token is provided -> inject it to every outgoing send request
        if token:
            listen(self, SendRequest, self.__inject_token)

        if metadata:
            listen(self, SendRequest, self.__inject_metadata)

    def clone(self, token: Optional[Union[str, Callable[["Channel"], str], Awaitable[str]]] = None) -> "Channel":
        return Channel(
            fqdn_or_ip=self.fqdn_or_ip,
            serial_number=self.serial_number,
            port=self.port,
            device_ca_cert=self.device_ca_cert,
            token=token if token != self.token else self.token,
            metadata=self.metadata,
            secure=self.secure,
        )

    async def __inject_token(self, event: SendRequest) -> None:
        # if token is method, call it to get token
        token = self.token
        if callable(token):
            token = token(self)
        if inspect.isawaitable(token):
            token = await token
        event.metadata["token"] = token

    async def __inject_metadata(self, event: SendRequest) -> None:
        event.metadata.update(self.metadata)

    async def __aenter__(self) -> "Channel":
        return self

    async def __aexit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        self.close()

    # Allow synchronous usage
    def __enter__(self) -> "Channel":
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        self.close()





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/hardware/services/compute_module.proto, blickfeld/hardware/services/device_operation.proto, blickfeld/hardware/services/identification.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import config as _config__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class IdentificationGetResponse(betterproto.Message):
    """Response message for the identification get command"""

    identification: "_config__.Identification" = betterproto.message_field(1)
    """The current identification"""


class ComputeModule(betterproto.ServiceStub):
    """
    This service is used to control & observe the state of the compute module.
    """

    async def async_reboot(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Reboots the CM4"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.hardware.services.ComputeModule/Reboot",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def reboot(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Reboots the CM4"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_reboot(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class DeviceOperation(betterproto.ServiceStub):
    """
    This service is used for starting and stopping the device operation.
    Normally, the device is started and stopped based on the client
    connections. If there is a client using the Qb2, the device will start and
    stay in operation until all clients disconnected. The start function of
    this service overrides this behaviour, i.e. when started using the start
    function the device will stay operational even though no clients are
    connected. This allows a faster retrieval of frames once a client connects
    as the startup phase is skipped.
    """

    async def async_start(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Method to start the device operation. This starts the operation ot the
        deflection mirrors and the laser triggering. When this call is used,
        the device will stay in operation until stop is called (independent//
        of clients being connected or not).
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.hardware.services.DeviceOperation/Start",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def start(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Method to start the device operation. This starts the operation ot the
        deflection mirrors and the laser triggering. When this call is used,
        the device will stay in operation until stop is called (independent//
        of clients being connected or not).
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_start(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_stop(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Method to stop the device operation. This will stop the deflection
        mirror operation and laser triggering.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.hardware.services.DeviceOperation/Stop",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def stop(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Method to stop the device operation. This will stop the deflection
        mirror operation and laser triggering.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_stop(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Identification(betterproto.ServiceStub):
    """
    The identification service provides methods to set and read out
    identification data of the Qb2 device.
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "IdentificationGetResponse":
        """Gets identification configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.hardware.services.Identification/Get",
            request,
            IdentificationGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "IdentificationGetResponse":
        """Gets identification configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



from grpc import StatusCode
from grpc.aio import AioRpcError


class BaseError(Exception):
    def __init__(self, code: StatusCode, message: str | None = None) -> None:
        super().__init__(code, message)
        #: :py:class:`~grpc.StatusCode` of the error
        self.code = code
        #: Error message
        self.message = message

    @staticmethod
    def create_from_exception(ex: Exception) -> "BaseError":
        if isinstance(ex, BaseError):
            return ex
        if isinstance(ex, AioRpcError):
            raise BaseError(
                ex.code(),
                ex.details(),
            )

        return BaseError(StatusCode.UNKNOWN, str(ex))



from . import config
from . import data
from . import geometry

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/base/options/access_control.proto, blickfeld/base/options/misc.proto, blickfeld/base/options/product_variant.proto, blickfeld/base/options/protocol_maturity.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class AccessControlLevel(betterproto.Enum):
    """
    Available access levels   The levels are ordered in ascending order, which
    means that each level also inherits the permissions of the levels above it.
    """

    LEVEL_UNSPECIFIED = 0
    """Access level is not specified"""

    LEVEL_PUBLIC = 1
    """
    Everyone can access   The corresponding entities can be accessed by anyone
    who has access to the interface of the device. The access can not be
    disabled by the user.
    """

    LEVEL_AUTHORIZED = 2
    """
    Only authorized clients can access   The corresponding entities can only be
    used by authorized clients. The access can be managed by clients with the
    ADMIN access level. If desired, the AUTHORIZED access level can be
    configured to act equally to the PUBLIC access level.
    """

    LEVEL_ADMIN = 3
    """
    Only administrative clients can access   The corresponding entities are
    mainly used to configure the accounts with AUTHORIZED & ADMIN access
    levels.
    """

    LEVEL_SUPPORT = 4
    """
    Only clients of the Blickfeld support can access   The corresponding
    entities can only be accessed by Blickfeld Support Tools & clients. The
    access tokens are bound to devices.
    """

    LEVEL_PRODUCTION = 5
    """
    Only clients in production can access   The corresponding entities are used
    for end-of-line & re-work processes.
    """

    LEVEL_DEVELOPER = 6
    """Only Blickfeld developers can access"""


class LifeTimeDiagnosticsType(betterproto.Enum):
    """
    Type of diagnostics.   Is used to aggregate multiple long term statistics.
    """

    TYPE_UNSPECIFIED = 0
    """Unspecified type."""

    TYPE_TEMPERATURE = 1
    """Temperature sensor or estimate."""

    TYPE_UTILIZATION = 2
    """Indicates how much of the available resource is used."""

    TYPE_RATE = 3
    """Frequency measurement or estimate."""

    TYPE_VOLTAGE = 4
    """Electrical voltage sensor or estimate."""


class ProtocolMaturityLevel(betterproto.Enum):
    """
    This enumeration describes the maturity of a protocol. Please refer to the
    individual values.   [IMPORTANT] APIs marked with the EXPERIMENTAL and
    DEPRECATED flags have to be handled with care. Please read through the
    definition.   Please report if the protocol maturity of a service, method,
    and field does not fit e.g. if an EXPERIMENTAL field is required in an
    application and production use case.
    """

    LEVEL_UNSPECIFIED = 0
    """
    Level is not set. Default level for services is EXPERIMENTAL. Default level
    for methods and fields is INHERITED.
    """

    LEVEL_EXPERIMENTAL = 1
    """
    APIs marked with this flag should be handled with care. They might not
    function as expected. Additionally it is not ensured that the APIs will
    ever be moved to the MAINTAINED level. They might be altered or removed in
    any upcoming release without further notice.
    """

    LEVEL_MAINTAINED = 2
    """
    APIs marked with this flag are actively maintained and released. They will
    not change their behavior without notices in the changelog. Breaking
    changes in a maintained APIs have to result in a major update of the
    module.
    """

    LEVEL_DEPRECATED = 3
    """
    APIs marked with this flag are no longer actively maintained. They will be
    removed in upcoming releases.
    """

    LEVEL_INHERITED = 4
    """The protocol maturity is inherited from the parent"""


class ProductVariant(betterproto.Enum):
    """The product variant specifies a Blickfeld Qb2 product variant."""

    PRODUCT_VARIANT_UNSPECIFIED = 0
    """
    Unspecified (default value) - placeholder for all other product variants.
    """

    PRODUCT_VARIANT_QBBASIC = 1
    """QbBasic - Basic & Core product variant."""

    PRODUCT_VARIANT_QBPROTECT = 2
    """
    QbProtect - Tailored product variant for security & protection
    applications.
    """

    PRODUCT_VARIANT_QBVOLUME = 3
    """
    QbVolume - Tailored product variant for stockpile management applications.
    """

    PRODUCT_VARIANT_QB2 = 4
    """
    Qb2 - Generic product variant allowing the exploration & discovery of new
    applicaton areas for smart lidar.
    """


@dataclass(eq=False, repr=False)
class AccessControl(betterproto.Message):
    """
    The Access Control message specifies the required access level and licenses
    for protocol entities. Protocol entities are usually gRPC services but can
    also be more fine grained on gRPC method and Protobuf field level.
    """

    level: "AccessControlLevel" = betterproto.enum_field(1)
    """Minimum access level required for entity"""

    read_only: bool = betterproto.bool_field(2)
    """True if the entity can be accessed with read-only permissions"""


@dataclass(eq=False, repr=False)
class LifeTimeDiagnostics(betterproto.Message):
    """
    This annotation is used by diagnostic modules to collect long term
    statistics for the annotated field.
    """

    type: "LifeTimeDiagnosticsType" = betterproto.enum_field(1)
    """Type"""


@dataclass(eq=False, repr=False)
class LinkData(betterproto.Message):
    """Message defining LinkData"""

    type: str = betterproto.string_field(1)
    directory: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ProtocolMaturity(betterproto.Message):
    """
    This message specifies the maturity of a protocol for customer- and
    production-facing APIs.
    """

    level: "ProtocolMaturityLevel" = betterproto.enum_field(1)
    """Simple level which describes maturity"""



import pathlib
import sys
from typing import (
    Generator,
    List,
    Set,
    Tuple,
    Union,
)

from blickfeld_qb2.betterproto.lib.google.protobuf import (
    DescriptorProto,
    EnumDescriptorProto,
    FileDescriptorProto,
    ServiceDescriptorProto,
)
from blickfeld_qb2.betterproto.lib.google.protobuf.compiler import (
    CodeGeneratorRequest,
    CodeGeneratorResponse,
    CodeGeneratorResponseFeature,
    CodeGeneratorResponseFile,
)

from .compiler import outputfile_compiler
from .models import (
    EnumDefinitionCompiler,
    FieldCompiler,
    MapEntryCompiler,
    MessageCompiler,
    OneOfFieldCompiler,
    OutputTemplate,
    PluginRequestCompiler,
    ServiceCompiler,
    ServiceMethodCompiler,
    is_map,
    is_oneof,
)


def traverse(
    proto_file: FileDescriptorProto,
) -> Generator[
    Tuple[Union[EnumDescriptorProto, DescriptorProto], List[int]], None, None
]:
    # Todo: Keep information about nested hierarchy
    def _traverse(
        path: List[int],
        items: Union[List[EnumDescriptorProto], List[DescriptorProto]],
        prefix: str = "",
    ) -> Generator[
        Tuple[Union[EnumDescriptorProto, DescriptorProto], List[int]], None, None
    ]:
        for i, item in enumerate(items):
            # Adjust the name since we flatten the hierarchy.
            # Todo: don't change the name, but include full name in returned tuple
            item.name = next_prefix = f"{prefix}_{item.name}"
            yield item, [*path, i]

            if isinstance(item, DescriptorProto):
                # Get nested types.
                yield from _traverse([*path, i, 4], item.enum_type, next_prefix)
                yield from _traverse([*path, i, 3], item.nested_type, next_prefix)

    yield from _traverse([5], proto_file.enum_type)
    yield from _traverse([4], proto_file.message_type)


def generate_code(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
    response = CodeGeneratorResponse()

    plugin_options = request.parameter.split(",") if request.parameter else []
    response.supported_features = CodeGeneratorResponseFeature.FEATURE_PROTO3_OPTIONAL

    request_data = PluginRequestCompiler(plugin_request_obj=request)
    # Gather output packages
    for proto_file in request.proto_file:
        output_package_name = proto_file.package
        if output_package_name not in request_data.output_packages:
            # Create a new output if there is no output for this package
            request_data.output_packages[output_package_name] = OutputTemplate(
                parent_request=request_data, package_proto_obj=proto_file
            )
        # Add this input file to the output corresponding to this package
        request_data.output_packages[output_package_name].input_files.append(proto_file)

        if (
            proto_file.package == "google.protobuf"
            and "INCLUDE_GOOGLE" not in plugin_options
        ):
            # If not INCLUDE_GOOGLE,
            # skip outputting Google's well-known types
            request_data.output_packages[output_package_name].output = False

    # Read Messages and Enums
    # We need to read Messages before Services in so that we can
    # get the references to input/output messages for each service
    for output_package_name, output_package in request_data.output_packages.items():
        for proto_input_file in output_package.input_files:
            for item, path in traverse(proto_input_file):
                read_protobuf_type(
                    source_file=proto_input_file,
                    item=item,
                    path=path,
                    output_package=output_package,
                )

    # Read Services
    for output_package_name, output_package in request_data.output_packages.items():
        for proto_input_file in output_package.input_files:
            for index, service in enumerate(proto_input_file.service):
                read_protobuf_service(proto_input_file, service, index, output_package)

    # Generate output files
    output_paths: Set[pathlib.Path] = set()
    for output_package_name, output_package in request_data.output_packages.items():
        if not output_package.output:
            continue

        # Add files to the response object
        output_path = pathlib.Path(*output_package_name.split("."), "__init__.py")
        output_paths.add(output_path)

        paths = map(
            lambda k: pathlib.Path(*k.split("."), "__init__.py"),
            request_data.output_packages.keys(),
        )

        content = (
            "\n".join(
                f"from . import {path.parent.name}"
                for path in paths
                if len(path.parents) > 1 and output_path.parent == list(path.parents)[1]
            )
            + "\n\n"
            + outputfile_compiler(output_file=output_package)
        )

        response.file.append(
            CodeGeneratorResponseFile(
                name=str(output_path),
                # Render and then format the output file
                content=content,
            )
        )

    # Make each output directory a package with __init__ file
    init_files = {
        directory.joinpath("__init__.py")
        for path in output_paths
        for directory in path.parents
        if not directory.joinpath("__init__.py").exists()
    } - output_paths

    for init_file in init_files:
        output_paths.add(init_file)

    for init_file in init_files:
        response.file.append(
            CodeGeneratorResponseFile(
                name=str(init_file),
                content="\n".join(
                    f"from . import {path.parent.name}"
                    for path in output_paths
                    if len(path.parents) > 1
                    and init_file.parent == list(path.parents)[1]
                ),
            )
        )

    for output_package_name in sorted(output_paths.union(init_files)):
        print(f"Writing {output_package_name}", file=sys.stderr)

    return response


def read_protobuf_type(
    item: DescriptorProto,
    path: List[int],
    source_file: "FileDescriptorProto",
    output_package: OutputTemplate,
) -> None:
    if isinstance(item, DescriptorProto):
        if item.options.map_entry:
            # Skip generated map entry messages since we just use dicts
            return
        # Process Message
        message_data = MessageCompiler(
            source_file=source_file, parent=output_package, proto_obj=item, path=path
        )
        for index, field in enumerate(item.field):
            if is_map(field, item):
                MapEntryCompiler(
                    source_file=source_file,
                    parent=message_data,
                    proto_obj=field,
                    path=path + [2, index],
                )
            elif is_oneof(field):
                OneOfFieldCompiler(
                    source_file=source_file,
                    parent=message_data,
                    proto_obj=field,
                    path=path + [2, index],
                )
            else:
                FieldCompiler(
                    source_file=source_file,
                    parent=message_data,
                    proto_obj=field,
                    path=path + [2, index],
                )
    elif isinstance(item, EnumDescriptorProto):
        # Enum
        EnumDefinitionCompiler(
            source_file=source_file, parent=output_package, proto_obj=item, path=path
        )


def read_protobuf_service(
    source_file: "FileDescriptorProto",
    service: ServiceDescriptorProto,
    index: int,
    output_package: OutputTemplate,
) -> None:
    service_data = ServiceCompiler(
        source_file=source_file,
        parent=output_package,
        proto_obj=service,
        path=[6, index],
    )
    for j, method in enumerate(service.method):
        ServiceMethodCompiler(
            source_file=source_file,
            parent=service_data,
            proto_obj=method,
            path=[6, index, 2, j],
        )





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/eye_safety/data/pulse_mode.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class PulseMode(betterproto.Enum):
    """
    The pulse mode specifies the system's laser energy that is emitted with
    each laser pulse. The different modes provide options to cater to different
    applications.
    """

    PULSE_MODE_UNSPECIFIED = 0
    """Unspecified pulse mode"""

    PULSE_MODE_A = 1
    """
    Default pulse mode providing a high pulse rate for a dense point cloud with
    sufficient detection rate for most use cases
    """

    PULSE_MODE_B = 2
    """
    Pulse mode with medium single pulse laser energy, balancing between A and C
    """

    PULSE_MODE_C = 3
    """
    Pulse mode with high single pulse laser energy, improving detection for far
    range or low reflectivity at the cost of pulse rate
    """





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_pipeline/config/background_subtraction.proto, blickfeld/percept_pipeline/config/classification.proto, blickfeld/percept_pipeline/config/clustering.proto, blickfeld/percept_pipeline/config/data_source.proto, blickfeld/percept_pipeline/config/disqualification.proto, blickfeld/percept_pipeline/config/history.proto, blickfeld/percept_pipeline/config/object_class.proto, blickfeld/percept_pipeline/config/object_size.proto, blickfeld/percept_pipeline/config/perception.proto, blickfeld/percept_pipeline/config/point_cloud_filter.proto, blickfeld/percept_pipeline/config/tracking.proto, blickfeld/percept_pipeline/config/zone_algorithm.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import (
    config as __base_config__,
    geometry as __base_geometry__,
)


class ObjectSize(betterproto.Enum):
    """The size of an object"""

    OBJECT_SIZE_UNSPECIFIED = 0
    """The zero value should not be used"""

    OBJECT_SIZE_SMALL = 1
    """Small object size: small animals, birds, etc"""

    OBJECT_SIZE_MEDIUM = 2
    """Medium object size: free standing human, single human, etc"""

    OBJECT_SIZE_LARGE = 3
    """Large object size: cars, group of people, large animals, etc"""


class ObjectClassTlsVehicle(betterproto.Enum):
    """
    Vehicle classification according to "Technische Lieferbedingungen für
    Streckenstationen " (TLS 2012) defined by the German regulatory
    "Bundesanstalt für Straßen- und Verkehrswesen" (bast).   The enum values
    directly reflect the class of the 8+1 classification.
    """

    TLS_VEHICLE_UNSPECIFIED = 0
    """The zero value should not be used."""

    TLS_VEHICLE_MOTORBIKE = 10
    """Motorbike (Krad)."""

    TLS_VEHICLE_CAR = 7
    """Standard passenger car (Pkw)."""

    TLS_VEHICLE_VAN = 11
    """Van (Lfw)."""

    TLS_VEHICLE_TRUCK = 3
    """Standard truck (Lkw)."""

    TLS_VEHICLE_TRUCK_WITH_TRAILER = 8
    """Truck towing a trailer (LkwA)."""

    TLS_VEHICLE_SEMI_TRAILER_TRUCK = 9
    """Semitrailer truck (Sattel-Kfz)."""

    TLS_VEHICLE_BUS = 5
    """Bus."""

    TLS_VEHICLE_CAR_WITH_TRAILER = 2
    """Car with trailer (PkwA)."""

    TLS_VEHICLE_OTHER = 6
    """Other or uncategorized object (nk Kfz)."""


class ClassificationModelType(betterproto.Enum):
    """Machine learning model types."""

    TYPE_UNSPECIFIED = 0
    """The zero value should not be used"""

    TYPE_SVM = 1
    """Support Vector Machine Model"""


class ClassificationModelFeatureGenerator(betterproto.Enum):
    """
    Enum specifying which feature generation method to use for ML model
    predictions.
    """

    FEATURE_GENERATOR_UNSPECIFIED = 0
    """The zero value should not be used"""

    FEATURE_GENERATOR_EIGVALS_DIMS = 1
    """
    Generate features from:  - Eigenvalues (λ1, λ2, λ3) of the 3D covariance
    matrix of the object’s point cloud.  - Object dimensions: height, width,
    and length. Resulting feature vector: [λ1, λ2, λ3, height, width, length]
    """

    FEATURE_GENERATOR_EIGVALS_DIMS_WITH_VARIANCE = 2
    """
    Generate features from:  - Eigenvalues (λ1, λ2, λ3) of the 3D covariance
    matrix.  - Object dimensions: height, width, and length.  - Per-axis
    variance: var_x, var_y, var_z. Resulting feature vector: [λ1, λ2, λ3,
    height, width, length, var_x, var_y, var_z]
    """


@dataclass(eq=False, repr=False)
class Clustering(betterproto.Message):
    """
    Algorithms for detecting objects as clustered points in the foreground
    scene.
    """

    db_scan: "ClusteringDbScan" = betterproto.message_field(1, group="clustering_type")
    """Use db-scan for clustering the foreground"""

    disable_percentile_bounding_boxes: bool = betterproto.bool_field(2)
    """
    Disabling the percentiles will lead to the dimensions being determined by
    the min and max points in each dimension.
    """


@dataclass(eq=False, repr=False)
class ClusteringDbScan(betterproto.Message):
    """
    Density-based spatial clustering algorithm using the euclidean distance
    between points
    """

    min_points: int = betterproto.uint32_field(1)
    """
    Minimum amount of points required to define an object-cluster   Reasonable
    default: 10
    """

    radius: float = betterproto.float_field(2)
    """
    Controls how far away points belonging to the same object-cluster can be.
    Reasonable default: 0.2
    """

    distance_dependent_radius: float = betterproto.float_field(3)
    """
    Multiply the given radius with the range of a point and add it to the
    radius. This significantly improves the clustering for close-range scenes.
    Reasonable default: 0.025
    """


@dataclass(eq=False, repr=False)
class BackgroundSubtraction(betterproto.Message):
    """
    Algorithms seperating the input point cloud into foreground and background.
    """

    mixture_of_gaussians: "BackgroundSubtractionMixtureOfGaussians" = (
        betterproto.message_field(1, group="background_subtraction_type")
    )
    """Use mixture of gaussians for foreground/background detection"""

    static: "BackgroundSubtractionStatic" = betterproto.message_field(
        3, group="background_subtraction_type"
    )
    """Use a static tree structure for foreground/background detection."""


@dataclass(eq=False, repr=False)
class BackgroundSubtractionMixtureOfGaussians(betterproto.Message):
    """
    Dynamic background subtraction using a mixture of gaussians. The algorithm
    automatically slowly updates the background model if the scene changes. The
    update rate is controlled via 'exponential_decay'.
    """

    num_initialization_frames: int = betterproto.uint32_field(1)
    """
    The amount of initially received frames which are used to build the
    background.   Reasonable default: 10
    """

    exponential_decay: float = betterproto.float_field(2)
    """
    Controls how fast objects switch between foreground and background.
    Exponential decay factor.   Reasonable default: 0.005
    """

    min_weight_threshold_for_background: float = betterproto.float_field(3)
    """
    Controls how much noise the background/foreground is expected to have.
    Reasonable default: 0.25
    """

    object_awareness: "BackgroundSubtractionMixtureOfGaussiansObjectAwareness" = (
        betterproto.message_field(5)
    )
    """Configure the object awareness of the background subtraction."""


@dataclass(eq=False, repr=False)
class BackgroundSubtractionMixtureOfGaussiansObjectAwareness(betterproto.Message):
    """
    The dynamic model update can depend on the detected objects. If enabled,
    the model update is freezed for areas around detected objects.   This can
    be very helpful to avoid that stalled objects, which stay in the scene for
    a while (e.g. parked cars) or objects which are following similar paths
    (e.g. persons in a hallway), cause the generation of an invalid background
    model. The dynamic model update is though still enabled for small objects
    or noise points (e.g. caused by vegetation).   To still allow the merging
    of objects into the scene background, simple rules are available which
    automatically re-initialize the background model for the area around a
    stall object. Also, objects can be explicitly merged into the background
    via separate API commands.
    """

    enabled: bool = betterproto.bool_field(1)
    """Enable object awareness"""

    auto_merge_object_duration: float = betterproto.float_field(2)
    """
    After an object has been tracked for the given duration, the area of the
    object is reinitialized.   If 0, the auto merging is disabled.   Reasonable
    default: 10 minutes
    """

    sizes: List["ObjectSize"] = betterproto.enum_field(3)
    """
    Only enable the object awareness for the given object sizes.   If empty, it
    is enabled for all sizes.   Reasonable default: OBJECT_SIZE_MEDIUM,
    OBJECT_SIZE_LARGE.
    """

    disable_for_in_background_objects: bool = betterproto.bool_field(4)
    """
    Disable for objects which are marked with the in_background property.
    This can be beneficial in scenes where the continuous background model
    update works better than the one-shot auto-merge re-initialization. Please
    note that the shadow detection will not work as expected when this is
    disabled.
    """


@dataclass(eq=False, repr=False)
class BackgroundSubtractionStatic(betterproto.Message):
    """
    Uses the first received frames to build a static background map of the
    environment. The algorithm requires the scene to only contain the static
    parts of the scene on the start of the processing. The background model is
    not updated automatically afterwards. The ResetBackground grpc method
    allows to update the static map of the environment.
    """

    num_initialization_frames: int = betterproto.uint32_field(1)
    """
    How many frames of the first received frames are used to build the
    background   Reasonable default: 10
    """

    voxel_size: float = betterproto.float_field(3)
    """
    Size of voxels used for voxelization of the background during
    initialization.   Default: 0.05 m
    """

    min_distance: float = betterproto.float_field(4)
    """
    Minimum spatial distance to background to detect points as foreground
    Default: 0.15 m
    """


@dataclass(eq=False, repr=False)
class ObjectClass(betterproto.Message):
    """ObjectClass represents classification types of detected objects."""

    tls_vehicle: "ObjectClassTlsVehicle" = betterproto.enum_field(1, group="type")
    """bast TLS 2012 vehicle classification 8+1"""


@dataclass(eq=False, repr=False)
class Classification(betterproto.Message):
    """
    Algorithms for classifying object into small, medium, large or predicting
    class of the object using machine learning models.
    """

    size: "ClassificationSize" = betterproto.message_field(1, group="type")
    """Rule-based classification using object size thresholds."""

    model: "ClassificationModel" = betterproto.message_field(2, group="type")
    """Model-based classification using a trained machine learning model."""


@dataclass(eq=False, repr=False)
class ClassificationSize(betterproto.Message):
    """
    The minimum size limits of the bounding box area used for classification.
    """

    bounding_box_surface: "ClassificationSizeBoundingBoxSurface" = (
        betterproto.message_field(1, group="type")
    )
    """Use object surface size for object classification."""


@dataclass(eq=False, repr=False)
class ClassificationSizeBoundingBoxSurface(betterproto.Message):
    """
    The maximum surface area between the bounding box's OXZ and OYZ planes is
    used to classify objects based on the limits specified in this message.
    """

    small_min_size: float = betterproto.float_field(1)
    """
    The surface area of the objects bounding box should be larger than this
    value to be considered a small object.
    """

    medium_min_size: float = betterproto.float_field(2)
    """
    The surface area of the objects bounding box should be larger than this
    value to be considered a medium object.
    """

    large_min_size: float = betterproto.float_field(3)
    """
    The surface area of the objects bounding box should be larger than this
    value to be considered a large object.
    """


@dataclass(eq=False, repr=False)
class ClassificationModel(betterproto.Message):
    """Configuration for model-based classification."""

    weights: bytes = betterproto.bytes_field(1)
    """Serialized model weights."""

    type: "ClassificationModelType" = betterproto.enum_field(2)
    """The type of the model."""

    normalization: "ClassificationModelNormalization" = betterproto.message_field(3)
    """The normalization parameters used to preprocess input features."""

    feature_generator: "ClassificationModelFeatureGenerator" = betterproto.enum_field(4)
    """The feature generator method used."""

    object_category: "ObjectClass" = betterproto.message_field(5)
    """The subset of classes to use."""

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(100)
    """
    Arbitrary metadata storage about model training, containing information
    about data used for training and optimal hyperparameters.
    """


@dataclass(eq=False, repr=False)
class ClassificationModelNormalization(betterproto.Message):
    """
    Configuration for feature normalization before feeding them into the model.
    """

    mean_coeffs: List[float] = betterproto.float_field(1)
    """
    The mean coefficients used to normalize each input feature. For each input
    feature: (x - mean) / std
    """

    std_coeffs: List[float] = betterproto.float_field(2)
    """The standard deviation coefficients used for normalization."""


@dataclass(eq=False, repr=False)
class DataSource(betterproto.Message):
    """
    The configuration storing the sources to retrieve the point cloud data for
    further processing.
    """

    qb2_setup: "DataSourceQb2Setup" = betterproto.message_field(
        3, group="data_source_type"
    )
    """Use Qb2 lidars to retrieve point clouds"""

    geolocation_of_map: "__base_config__.Geolocation" = betterproto.message_field(4)
    """
    Geolocation of data source. This references the map coordinate system of
    this data source to the world. Individual lidars are referenced to the map
    coordinate system via their `map_from_lidar` transform.   [NOTE] If the
    map_from_lidar transform is not an identity transformation, this is not the
    geolocation of an individual sensor.
    """

    swarm: bool = betterproto.bool_field(5)
    """
    Experimental mode which enables configuration synchronization and
    distributed processing.   All processing-related configuration, e.g. scan
    pattern and zones, are automatically synchronized between all members in
    the sensor setup. Also the sensor setup itself is replicated on all
    sensors.   The object lists of all sensors are fused and tracked on every
    member in the swarm. Thus, also all object-based security zones are
    evaluated on every sensor. This leads to a high availablity and redundancy
    without requiring a master processing unit would be a single point of
    failure.
    """

    allow_unavailable_lidars: bool = betterproto.bool_field(6)
    """
    Allow that devices in the setup are temporarily not available or failed.
    Re-connection attempts are performed continously. A warning with the list
    of unavailable sensors is raised. If all sensors are unavailable, the
    pipeline will fail.
    """


@dataclass(eq=False, repr=False)
class DataSourceQb2(betterproto.Message):
    """Configuration for retrieving point clouds from a Qb2"""

    fqdn: str = betterproto.string_field(1)
    """
    The fully qualified domain name of the Qb2 device. For on-device
    processing/configuration, 'fqdn' can be set to an empty string to request
    the data of the current device.
    """

    port: int = betterproto.uint32_field(5)
    """
    The gRPC port of the Qb2 device. If unset the default gRPC ports 50051
    (unencrypted) or 55551 (TLS) will be used.
    """

    map_from_lidar: "__base_geometry__.Transform" = betterproto.message_field(2)
    """
    Transformation from the lidar frame to the map frame. If this transform is
    not set, this will be interpreted as identify transformation. Transforming
    all point cloud topics into the map frame results in a globally consistent
    combined point cloud in map frame.
    """

    application_key: str = betterproto.string_field(3)
    """
    Application key with authorized access level. If set, the field
    serial_number has to be set as well. This field is required if user-
    management is enabled on externally connected devices.
    """

    serial_number: str = betterproto.string_field(4)
    """
    Serial number. If set, the connection will be secured with TLS. This field
    is required if user-management is enabled on externally connected devices.
    """

    disabled: bool = betterproto.bool_field(6)
    """
    Disable fetching data from this sensor, setting this flag to `true`
    disables the evaluation of the entire sensor point cloud
    """

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(100)
    """Arbitrary metadata storage for client applications"""


@dataclass(eq=False, repr=False)
class DataSourceTrigger(betterproto.Message):
    """
    Configuration of Trigger Mode. In trigger mode the processing pipeline will
    reduce the frequency at which point clouds and state lists are getting
    computed to reduce the amount of data transmitted between the processing
    device and configured 'lidars'.
    """

    frequency: float = betterproto.float_field(1)
    """
    The frequency at which data is getting requested from the lidars and
    outputs are generated.
    """


@dataclass(eq=False, repr=False)
class DataSourceQb2Setup(betterproto.Message):
    """The configuration of a Qb2 setup of the LiDAR devices"""

    lidars: List["DataSourceQb2"] = betterproto.message_field(1)
    """the qb2 lidars to get the point clouds from"""

    trigger: "DataSourceTrigger" = betterproto.message_field(2)
    """
    If the field is set trigger mode is enabled. If the field is not set, the
    pipeline outputs are computed with the frequency of configured lidars, i.e.
    the scan pattern configured on those.
    """


@dataclass(eq=False, repr=False)
class Disqualification(betterproto.Message):
    """
    Algorithm for detecting disqualifying effects for the sensor e.g. fog.
    """

    disabled: bool = betterproto.bool_field(1)
    """Disable disqualification checking."""

    fog_photon_count_threshold: float = betterproto.float_field(2)
    """
    Threshold for the detection of fog.   If the difference in photon count
    mean between the current frame and the background divided by the background
    mean exceeds this threshold, a disqualification alarm is raised.   Default:
    0.35
    """


@dataclass(eq=False, repr=False)
class History(betterproto.Message):
    """
    Configuration message for the event and motion replay history module.
    """

    disable: bool = betterproto.bool_field(1)
    """Disable non-volatile history."""


@dataclass(eq=False, repr=False)
class PointCloudFilter(betterproto.Message):
    """Algorithm for filtering point clouds"""

    radius_outlier: "PointCloudFilterRadiusOutlier" = betterproto.message_field(
        1, group="point_cloud_filter_type"
    )
    """Use radius outlier filter for noise reduction on point clouds"""


@dataclass(eq=False, repr=False)
class PointCloudFilterRadiusOutlier(betterproto.Message):
    """Filter points based on the number of neighbors in a certain radius"""

    min_neighbor_points: int = betterproto.uint32_field(1)
    """Minimum number of neighbors a point has to have to be retained"""

    neighbor_radius: float = betterproto.double_field(2)
    """All points within this radius are considered neighbors of a point"""


@dataclass(eq=False, repr=False)
class Tracking(betterproto.Message):
    """Algorithms for tracking the objects generated from the clustering"""

    disable: bool = betterproto.bool_field(4)
    """Disable tracking. On by default."""

    sensitivity: float = betterproto.float_field(1)
    """
    Higher sensitivity will make the tracker confirm the object faster
    Reasonable default: 0.85
    """

    acceleration_noise: float = betterproto.float_field(2)
    """
    The acceptance range of accelerations for the object to be tracked
    Reasonable default: 0.5
    """

    survival_probability: float = betterproto.float_field(3)
    """
    Higher probability will allow the object to be followed longer in the
    absence of observations   Reasonable default: 1
    """

    path_resolution: float = betterproto.float_field(5)
    """
    The distance between saved object positions in the track path history
    Default value: 1 meter
    """


@dataclass(eq=False, repr=False)
class Perception(betterproto.Message):
    """
    The configuration of perception algorithms used to run e.g. security zones
    """

    background_subtraction: "BackgroundSubtraction" = betterproto.message_field(1)
    """Extract the foreground point cloud of the scene"""

    foreground_point_cloud_filters: List["PointCloudFilter"] = (
        betterproto.message_field(2)
    )
    """Filter the foreground point cloud"""

    clustering: "Clustering" = betterproto.message_field(8)
    """Segment the foreground point cloud into clusters"""

    tracking: "Tracking" = betterproto.message_field(9)
    """Track the objects generated from the clustering"""

    classification: "Classification" = betterproto.message_field(10)
    """Classify object into small, medium and large objects"""

    history: "History" = betterproto.message_field(11)
    """Keep a history of perception results"""

    disqualification: "Disqualification" = betterproto.message_field(12)
    """Detect disqualifying effects in the point cloud caused by weather"""


@dataclass(eq=False, repr=False)
class ZoneAlgorithm(betterproto.Message):
    """An algorithm that monitors the area within and around a zone."""

    name: str = betterproto.string_field(1)
    """The user readable zone algorithm name"""

    uuid: str = betterproto.string_field(2)
    """The uuid of the zone algorithm (uuid4)"""

    shape: "__base_geometry__.Shape" = betterproto.message_field(3)
    """
    Geometric shape (usually a Box) in which the algorithm detects certain
    behaviors/scenarios
    """

    disabled: bool = betterproto.bool_field(13)
    """
    Disable this zone, setting this flag to `true` disables the evaluation of
    the zone
    """

    geolocation: "ZoneAlgorithmGeolocation" = betterproto.message_field(15)
    """
    Geolocation of the zone.   This field is read only and only set when the
    geolocation is configured in the data source.
    """

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(100)
    """Arbitrary metadata storage for client applications"""

    volume: "ZoneAlgorithmVolume" = betterproto.message_field(9, group="algorithm_type")
    """Run volume monitoring within the zone"""

    security: "ZoneAlgorithmSecurity" = betterproto.message_field(
        7, group="algorithm_type"
    )
    """Run alarm detection within the zone"""

    exclusion: "ZoneAlgorithmExclusion" = betterproto.message_field(
        10, group="algorithm_type"
    )
    """Run exclusion zone"""

    object_based_security: "ZoneAlgorithmObjectBasedSecurity" = (
        betterproto.message_field(11, group="algorithm_type")
    )
    """Run object-based alarm detection within the zone"""

    occupancy: "ZoneAlgorithmOccupancy" = betterproto.message_field(
        12, group="algorithm_type"
    )
    """Run occupancy zone"""

    traversal_security: "ZoneAlgorithmTraversalSecurity" = betterproto.message_field(
        14, group="algorithm_type"
    )
    """Run traversal zone"""

    traffic_lane: "ZoneAlgorithmTrafficLane" = betterproto.message_field(
        16, group="algorithm_type"
    )
    """Run traffic monitoring within the zone"""


@dataclass(eq=False, repr=False)
class ZoneAlgorithmGeolocation(betterproto.Message):
    """The geolocation of the zone based on the map and the zone position."""

    center: "__base_config__.Geolocation" = betterproto.message_field(1)
    """The geolocation of the center of the zone."""

    corners: List["__base_config__.Geolocation"] = betterproto.message_field(2)
    """The geolocation of the corners of the zone."""


@dataclass(eq=False, repr=False)
class ZoneAlgorithmVolume(betterproto.Message):
    """
    Algorithm that measure the volume of a zone   The volume of a zone is
    approximated by computing a 'volume map' from the input point clouds. The
    volume map discretize the zone into 'tiles' in the xy-plane with a side-
    length of 'resolution'. Each tile contains the height information about the
    points that fall into this tile. The volume of the volume map can be
    calculated by adding up the volumes of the tiles in the volume map.
    """

    resolution: float = betterproto.float_field(1)
    """
    Grid resolution per tile used to measure the volume.   Reasonable default:
    0.1
    """

    empty_volume: float = betterproto.float_field(2)
    """
    The measured volume while area covered by the zone is considered 'empty'.
    The value can be set using TareVolume method while the area of the zone is
    empty.   Reasonable default: 0
    """

    disable_when_occupied_zone_uuids: List[str] = betterproto.string_field(4)
    """
    Link volume zone to occupancy zones (by uuid).   Disables the volume zone
    state when one of the occupancy zones is occupied.
    """

    angle_of_repose: float = betterproto.float_field(5)
    """
    Angle of repose.   Steepest angle of descent when the measured material is
    piled. See https://en.wikipedia.org/wiki/Angle_of_repose.
    """

    minimum_coverage: float = betterproto.float_field(6)
    """
    Minimum coverage value for zone.   When the estimated coverage is below the
    configured threshold, the state output will be disabled.   This can be used
    to automatically disable zones when the viewport to the zone area is
    blocked or the sensor is temporarily failed.   Default: 0
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmSecurity(betterproto.Message):
    """
    Algorithm that triggers an alarm if movement was detected in the zone
    """

    min_points: int = betterproto.uint32_field(1)
    """
    Minimum number of (foreground) points in the security zone to trigger the
    alarm. Reasonable default: 10
    """

    max_points: int = betterproto.uint32_field(2)
    """
    Maximum number of (foreground) points in the security zone that still
    triggers an alarm. More points will disable the alarm. This logic can be
    disabled by setting max_points to '0'. Default: 0
    """

    minimum_intruding_duration: float = betterproto.float_field(3)
    """
    Minimum active intrusion duration before alarm is triggered.   Default: 0
    """

    minimum_intrusion_confidence: float = betterproto.float_field(4)
    """
    Minimum intrusion confidence before alarm is triggered.   Only applicable
    when the minimum intruding duration is configured.   Default: 1
    """

    hold_duration: float = betterproto.float_field(5)
    """
    Alarm is kept active for the given duration to reduce jitter.   Default: 0
    """

    disable_when_occupied_zone_uuids: List[str] = betterproto.string_field(6)
    """
    Link security zone to occupancy zones (by uuid).   Disables the security
    zone state and alarms when one of the occupancy zones is occupied.
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmObjectBasedSecurity(betterproto.Message):
    """
    Algorithm that triggers an alarm when an object of one of the selected
    sizes is in the zone
    """

    alarm_sizes: List["ObjectSize"] = betterproto.enum_field(1)
    """
    Selection of sizes which trigger an alarm. Each size in the array will only
    trigger an alarm for the interval of that size.
    """

    minimum_intruder_lifetime: float = betterproto.float_field(6)
    """Minimum lifetime of object before alarm is triggered.   Default: 0"""

    minimum_intruder_track_length: float = betterproto.float_field(7)
    """
    Minimum track length of object before alarm is triggered.   Default: 0
    """

    minimum_intruding_duration: float = betterproto.float_field(8)
    """
    Minimum active intrusion duration before alarm is triggered.   Default: 0
    """

    objects_in_background: bool = betterproto.bool_field(4)
    """
    Trigger alarm for objects with the "in_background" property.   Enabling
    this might increase the false alarm rate.
    """

    minimum_intrusion_confidence: float = betterproto.float_field(9)
    """
    Minimum intrusion confidence before alarm is triggered.   Only applicable
    when the minimum intruding duration is configured.   Default: 1
    """

    hold_duration: float = betterproto.float_field(10)
    """
    Alarm is kept active for the given duration to reduce jitter.   Default: 0
    """

    intrusion_on_partial_entry: bool = betterproto.bool_field(11)
    """
    Detect an object as intruder if any part of it enters the zone. Only the
    center point of the object is considered when disabled.   Enabling this
    might increase the false alarm rate.
    """

    disable_when_occupied_zone_uuids: List[str] = betterproto.string_field(12)
    """
    Link security zone to occupancy zones (by uuid).   Disables the security
    zone state and alarms when one of the occupancy zones is occupied.
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmExclusion(betterproto.Message):
    """Algorithm that excludes points from the input point cloud"""

    pass


@dataclass(eq=False, repr=False)
class ZoneAlgorithmOccupancy(betterproto.Message):
    """Algorithm that detects if a zone is occupied"""

    min_points: int = betterproto.uint32_field(1)
    """
    Minimum number of points to mark the zone as occupied.   Reasonable
    default: 10
    """

    adaptive_baseline_time_constant: float = betterproto.float_field(2)
    """
    If set, the min_points is not compared against zero but against an adaptive
    baseline. The baseline is calculated with an exponential filter with the
    zone point count as input.   The given time constant is used to calculate
    the alpha for the exponential filter. After a restart, the baseline is
    initialized with the current zone point count.   Reasonable default: 3600s
    (1 hour)
    """

    voxel_size: float = betterproto.float_field(3)
    """
    The point cloud inside the zone is down sampled with a voxel grid. This is
    done to achieve a distance-independent parametrization. The size of a voxel
    cell can be configured with this parameter.   Default: 0.05m
    """

    invert: bool = betterproto.bool_field(4)
    """
    Invert the state of the zone   This might be useful in situations where
    e.g. a static crane should be detected in its home position.
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmTraversalSecurity(betterproto.Message):
    """Algorithm that detects if objects are traversing through the zone."""

    alarm_sizes: List["ObjectSize"] = betterproto.enum_field(1)
    """
    Selection of sizes which trigger an alarm.  An alarm will only be triggered
    by objects of sizes specified in the array.
    """

    hold_duration: float = betterproto.float_field(2)
    """
    Alarm is kept active for the given duration to reduce jitter.   Default: 0
    """

    disable_when_occupied_zone_uuids: List[str] = betterproto.string_field(3)
    """
    Link security zone to occupancy zones (by uuid).   Disables the security
    zone state and alarms when one of the occupancy zones is occupied.
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmTrafficLane(betterproto.Message):
    """Algorithm that monitors traffic moving through the zone."""

    expected_velocity: float = betterproto.float_field(1)
    """Expected velocity of the objects inside the traffic zone."""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/base/config/geolocation.proto, blickfeld/base/config/range.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class Range(betterproto.Message):
    """
    This section defines ranges for numerical types. They can be used to define
    minimum and maximum values.
    """

    pass


@dataclass(eq=False, repr=False)
class RangeUint32(betterproto.Message):
    """An unsigned integer range"""

    minimum: int = betterproto.uint32_field(1)
    """An unsigned integer minimum value"""

    maximum: int = betterproto.uint32_field(2)
    """An unsigned integer maximum value"""


@dataclass(eq=False, repr=False)
class RangeInt32(betterproto.Message):
    """A signed integer range"""

    minimum: int = betterproto.int32_field(1)
    """A signed integer minimum value"""

    maximum: int = betterproto.int32_field(2)
    """A signed integer maximum value"""


@dataclass(eq=False, repr=False)
class RangeFloat(betterproto.Message):
    """A single precision float range"""

    minimum: float = betterproto.float_field(1)
    """A single precision float minimum"""

    maximum: float = betterproto.float_field(2)
    """A single precision float maximum"""


@dataclass(eq=False, repr=False)
class RangeDouble(betterproto.Message):
    """A double precision float range"""

    minimum: float = betterproto.double_field(1)
    """A double precision float minimum"""

    maximum: float = betterproto.double_field(2)
    """A double precision float maximum"""


@dataclass(eq=False, repr=False)
class Geolocation(betterproto.Message):
    """
    Geolocation in the geographic coordinate system. Is used to geo-reference
    device coordinate systems and produced data.
    """

    latitude: float = betterproto.float_field(1)
    """Latitude angle"""

    longitude: float = betterproto.float_field(2)
    """Longitude angle"""

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(100)
    """Arbitrary metadata storage for client applications"""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/onvif/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Describes the health of the ONVIF module"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level health state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for a given state. Only set if state is not STATE_OK"""

    clients: Dict[str, "HealthClient"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Served clients (key is unique subscription uuid)"""


@dataclass(eq=False, repr=False)
class HealthTopic(betterproto.Message):
    """Notification subscription topic"""

    number_of_raised_events: int = betterproto.uint32_field(1)
    """
    number of raised events corresponding to the topic name (counted on the
    raising edge)
    """


@dataclass(eq=False, repr=False)
class HealthClient(betterproto.Message):
    """Describes the connected VMS client"""

    ip: str = betterproto.string_field(1)
    """IP address of the client"""

    uptime: int = betterproto.uint32_field(2)
    """
    Time since the client has been actively pulling the ONVIF notifications
    """

    subscribed_topics: Dict[str, "HealthTopic"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Active subscription topics (key: topic name)"""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/flow/config/flow.proto, blickfeld/flow/config/node_red_json.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class Flow(betterproto.Message):
    """
    Blickfeld Flow configuration   This container is used to control high-level
    attributes of NodeRed flows.
    """

    name: str = betterproto.string_field(1)
    """Human readable label of the flow"""

    disabled: bool = betterproto.bool_field(2)
    """Is it active?"""

    locked: bool = betterproto.bool_field(3)
    """Is it read only?"""

    is_subflow: bool = betterproto.bool_field(4)
    """Is it a subflow?"""

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(5)
    """Other arbitrary metadata"""

    nodes: List["betterproto_lib_google_protobuf.Struct"] = betterproto.message_field(6)
    """All nodes contained in the flow"""


@dataclass(eq=False, repr=False)
class NodeRedJson(betterproto.Message):
    """Simple container for the node red import / export format"""

    nodes: List["betterproto_lib_google_protobuf.Struct"] = betterproto.message_field(1)
    """List of JSON nodes"""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_processing/services/event.proto, blickfeld/percept_processing/services/health.proto, blickfeld/percept_processing/services/history.proto, blickfeld/percept_processing/services/objects.proto, blickfeld/percept_processing/services/point_cloud.proto, blickfeld/percept_processing/services/states.proto, blickfeld/percept_processing/services/volume_map.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from ...base import geometry as __base_geometry__
from ...core_processing import data as __core_processing_data__
from ...percept_pipeline import data as __percept_pipeline_data__
from .. import data as _data__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class EventStreamRequest(betterproto.Message):
    """
    Request for stream method   NOTE: Only a limited amount of events is
    persisted.
    """

    id: str = betterproto.string_field(1)
    """Identifier from which the stream should start."""

    reverse: bool = betterproto.bool_field(2)
    """
    Stream events in reverse. This fetches events from persistent database.
    The events are emitted in batches. When no stream response is returned, no
    more events are available.
    """


@dataclass(eq=False, repr=False)
class EventStreamResponse(betterproto.Message):
    """Response to stream method"""

    id: str = betterproto.string_field(1)
    """
    Unique identifier of event.   The ID has the format: $timestamp-$revision.
    $timestamp is a monotonic counter in seconds. $revision is usually 0 but
    incremented when there a multiple events per second.
    """

    event: "_data__.Event" = betterproto.message_field(2)
    """Event payload."""


@dataclass(eq=False, repr=False)
class EventGetRequest(betterproto.Message):
    """Request for Get method"""

    id: str = betterproto.string_field(1)
    """
    Identifier of the event; see EventStreamRequest and EventStreamResponse for
    more details
    """


@dataclass(eq=False, repr=False)
class EventGetResponse(betterproto.Message):
    """Response to Get method"""

    event: "_data__.Event" = betterproto.message_field(2)
    """Event payload"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """
    A response for getting a stream of health messages about the pipeline
    """

    health: "__percept_pipeline_data__.Health" = betterproto.message_field(1)
    """
    The current health message with information about the state of the pipeline
    """


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """A response for getting an health message about the pipeline"""

    health: "__percept_pipeline_data__.Health" = betterproto.message_field(1)
    """
    The current health message with information about the state of the pipeline
    """


@dataclass(eq=False, repr=False)
class HistoryStreamRangeRequest(betterproto.Message):
    """Request for stream range method"""

    start_id: str = betterproto.string_field(1)
    """ID where the stream should start"""

    end_id: str = betterproto.string_field(2)
    """ID where the stream should stop"""


@dataclass(eq=False, repr=False)
class HistoryStreamRangeResponse(betterproto.Message):
    """Response for stream range method"""

    objects: "_data__.Objects" = betterproto.message_field(1)
    """Historical objects"""

    states: "_data__.States" = betterproto.message_field(2)
    """Historical states"""


@dataclass(eq=False, repr=False)
class ObjectsStreamRequest(betterproto.Message):
    """Request message for objects stream"""

    untracked: bool = betterproto.bool_field(1)
    """Fetch object list without tracking"""


@dataclass(eq=False, repr=False)
class ObjectsStreamResponse(betterproto.Message):
    """
    A response for getting a stream of objects messages from the running
    pipeline
    """

    objects: "_data__.Objects" = betterproto.message_field(1)
    """The current objects detected in the scene."""


@dataclass(eq=False, repr=False)
class PointCloudStreamRequest(betterproto.Message):
    """
    A request to receive a stream of point cloud messages. Different point
    clouds can be streamed depending on the set fields: - combined point cloud
    (the full point cloud of all the available devices combined) in the map
    coordinate system - the foreground of the combined point cloud in the map
    coordinate system - the complete raw point cloud coming from one device,
    but only in map coordinate system.
    """

    point_cloud_type: "__percept_pipeline_data__.PointCloudType" = (
        betterproto.enum_field(1)
    )
    """The type of point cloud to stream"""

    coordinate_system: "__percept_pipeline_data__.CoordinateSystem" = (
        betterproto.enum_field(2)
    )
    """
    The coordinate system the point cloud should be expressed in (local or
    transformed/map)
    """

    fqdn: str = betterproto.string_field(3, group="point_cloud_source")
    """fqdn of the data source to get the point cloud from"""

    crop: "__base_geometry__.Shape" = betterproto.message_field(5)
    """If set, point cloud is cropped to given shape"""


@dataclass(eq=False, repr=False)
class PointCloudStreamResponse(betterproto.Message):
    """A response for getting a stream of point cloud messages"""

    point_cloud: "__core_processing_data__.Frame" = betterproto.message_field(1)
    """The required point cloud, based on the fields set in the request"""


@dataclass(eq=False, repr=False)
class StatesStreamResponse(betterproto.Message):
    """
    A response for getting a stream of states messages from the running
    pipeline
    """

    states: "_data__.States" = betterproto.message_field(1)
    """The current states with the information detected in the scene."""


@dataclass(eq=False, repr=False)
class VolumeMapStreamRequest(betterproto.Message):
    """A request to receive a stream of state list messages"""

    zone_uuid: str = betterproto.string_field(2)
    """Volume zone uuid"""


@dataclass(eq=False, repr=False)
class VolumeMapStreamResponse(betterproto.Message):
    """
    A response for getting a stream of volume map messages from the running
    pipeline
    """

    volume_map: "_data__.VolumeMap" = betterproto.message_field(1)
    """The current volume map of the requested zone"""


class Event(betterproto.ServiceStub):
    """Service for actionable events"""

    async def async_stream(
        self,
        *,
        id: str = "",
        reverse: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["EventStreamResponse"]:
        """A method to get a stream of events."""

        request = EventStreamRequest()
        request.id = id
        request.reverse = reverse

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.Event/Stream",
            request,
            EventStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        id: str = "",
        reverse: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["EventStreamResponse"]:
        """A method to get a stream of events."""

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            id=id,
            reverse=reverse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get(
        self,
        *,
        id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "EventGetResponse":
        """Method to get one Event by its id"""

        request = EventGetRequest()
        request.id = id

        return await self._unary_unary(
            "/blickfeld.percept_processing.services.Event/Get",
            request,
            EventGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        *,
        id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "EventGetResponse":
        """Method to get one Event by its id"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                id=id,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """
    An RPC service to request information about the state of the pipeline and
    of the module.
    """

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        A method to get a stream of health messages, containing information
        about the state of the pipeline
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        A method to get a stream of health messages, containing information
        about the state of the pipeline
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """
        A method to get one health message, containing information about the
        state of the pipeline
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_processing.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """
        A method to get one health message, containing information about the
        state of the pipeline
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class History(betterproto.ServiceStub):
    """
    The history service provides historical data. The data is captured event-
    based e.g. when an intrusion occurs. The capture window is +/- 10 seconds
    around the event. The data is intended for event analysis and fine tuning
    of config.   [NOTE] The data is stored in volatile memory thus it is not
    persisted over a power reset of the device. Additionally, only a limited
    amount is kept.
    """

    async def async_stream_range(
        self,
        *,
        start_id: str = "",
        end_id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["HistoryStreamRangeResponse"]:
        """
        Stream historical data in the given id range.   Raises a NOT_FOUND
        error when no historical data in the given range is available.
        """

        request = HistoryStreamRangeRequest()
        request.start_id = start_id
        request.end_id = end_id

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.History/StreamRange",
            request,
            HistoryStreamRangeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream_range(
        self,
        *,
        start_id: str = "",
        end_id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["HistoryStreamRangeResponse"]:
        """
        Stream historical data in the given id range.   Raises a NOT_FOUND
        error when no historical data in the given range is available.
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream_range(
            start_id=start_id,
            end_id=end_id,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Objects(betterproto.ServiceStub):
    """
    An RPC service to request a stream of objects messages. If a pipeline is
    already running, it will be used to stream the data. If no pipeline is
    running, it will be started first and then the stream will start. NOTE: in
    this second case, the pipeline will also be automatically stopped once
    there are no more clients requesting any stream type from the module.
    """

    async def async_stream(
        self,
        *,
        untracked: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["ObjectsStreamResponse"]:
        """A method to get a stream of objects messages from the pipeline"""

        request = ObjectsStreamRequest()
        request.untracked = untracked

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.Objects/Stream",
            request,
            ObjectsStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        untracked: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["ObjectsStreamResponse"]:
        """A method to get a stream of objects messages from the pipeline"""

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            untracked=untracked,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class PointCloud(betterproto.ServiceStub):
    """
    An RPC service to request a stream of point cloud messages. If a pipeline
    is already running, it will be used to stream the data. If no pipeline is
    running, it will be started first and then the stream will start. NOTE: in
    this second case, the pipeline will also be automatically stopped once the
    this second case, the pipeline will also be automatically stopped once
    there are no more clients requesting any stream type from the module.
    """

    async def async_stream(
        self,
        *,
        point_cloud_type: "__percept_pipeline_data__.PointCloudType" = 0,
        coordinate_system: "__percept_pipeline_data__.CoordinateSystem" = 0,
        fqdn: str = "",
        crop: "__base_geometry__.Shape" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["PointCloudStreamResponse"]:
        """
        A method to get a stream of point cloud messages from the pipeline. It
        is possible to stream different point clouds, depending on the fields
        set in the request
        """

        request = PointCloudStreamRequest()
        request.point_cloud_type = point_cloud_type
        request.coordinate_system = coordinate_system
        request.fqdn = fqdn
        if crop is not None:
            request.crop = crop

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.PointCloud/Stream",
            request,
            PointCloudStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        point_cloud_type: "__percept_pipeline_data__.PointCloudType" = 0,
        coordinate_system: "__percept_pipeline_data__.CoordinateSystem" = 0,
        fqdn: str = "",
        crop: "__base_geometry__.Shape" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["PointCloudStreamResponse"]:
        """
        A method to get a stream of point cloud messages from the pipeline. It
        is possible to stream different point clouds, depending on the fields
        set in the request
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            point_cloud_type=point_cloud_type,
            coordinate_system=coordinate_system,
            fqdn=fqdn,
            crop=crop,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class States(betterproto.ServiceStub):
    """
    An RPC service to request a stream of states messages. If a pipeline is
    already running, it will be used to stream the data. If no pipeline is
    running, it will be started first and then the stream will start. NOTE: in
    this second case, the pipeline will also be automatically stopped once
    there are no more clients requesting any stream type from the module.
    """

    async def async_stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["StatesStreamResponse"]:
        """A method to get a stream of states messages from the pipeline"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.States/Stream",
            request,
            StatesStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["StatesStreamResponse"]:
        """A method to get a stream of states messages from the pipeline"""

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class VolumeMap(betterproto.ServiceStub):
    """An RPC service to request a stream of volume map messages."""

    async def async_stream(
        self,
        *,
        zone_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["VolumeMapStreamResponse"]:
        """
        A method to get a stream of volume map messages of one volume zone from
        the pipeline
        """

        request = VolumeMapStreamRequest()
        request.zone_uuid = zone_uuid

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.VolumeMap/Stream",
            request,
            VolumeMapStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        zone_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["VolumeMapStreamResponse"]:
        """
        A method to get a stream of volume map messages of one volume zone from
        the pipeline
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            zone_uuid=zone_uuid,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/video/config/visualization.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import geometry as __base_geometry__


class VisualizationRotation(betterproto.Enum):
    """
    Defines the clockwise rotation value for the frame in the video stream
    """

    ROTATION_UNSPECIFIED = 0
    """no video frame rotation"""

    ROTATION_90 = 1
    """rotate video frame 90 degrees"""

    ROTATION_180 = 2
    """rotate video frame 180 degrees"""

    ROTATION_270 = 3
    """rotate video frame 270 degrees"""


@dataclass(eq=False, repr=False)
class Visualization(betterproto.Message):
    """
    Message representing the visuals configuration of the RTSP video stream
    """

    frame: "VisualizationFrame" = betterproto.message_field(1)
    """frame configuration"""

    camera: "VisualizationCamera" = betterproto.message_field(3)
    """camera configuration"""


@dataclass(eq=False, repr=False)
class VisualizationColor(betterproto.Message):
    """
    RGB color used for coloring various elements and objects inside RTSP video
    frame
    """

    red: float = betterproto.float_field(1)
    """The amount of red in the color as a value in the interval [0, 1]."""

    green: float = betterproto.float_field(2)
    """The amount of green in the color as a value in the interval [0, 1]."""

    blue: float = betterproto.float_field(3)
    """The amount of blue in the color as a value in the interval [0, 1]."""

    alpha: float = betterproto.float_field(4)
    """
    The alpha channel (transparency) in the interval [0, 1]. 0 means
    transparent
    """


@dataclass(eq=False, repr=False)
class VisualizationFrame(betterproto.Message):
    """
    Visual configuration of point cloud frame. Element coloring can be disabled
    by unsetting the corresponding color.
    """

    background: "VisualizationColor" = betterproto.message_field(1)
    """color used to mark background point"""

    foreground: "VisualizationColor" = betterproto.message_field(2)
    """color used to mark foreground point - gray"""

    rotation: "VisualizationRotation" = betterproto.enum_field(7)
    """clockwise rotation value applied to the frame"""


@dataclass(eq=False, repr=False)
class VisualizationCamera(betterproto.Message):
    """Configuration of the camera"""

    position: "__base_geometry__.Vector3" = betterproto.message_field(1)
    """camera position - where camera is located"""

    target: "__base_geometry__.Vector3" = betterproto.message_field(2)
    """camera target - target at which camera looks"""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_processing/data/detected_object.proto, blickfeld/percept_processing/data/disqualification.proto, blickfeld/percept_processing/data/event.proto, blickfeld/percept_processing/data/flag.proto, blickfeld/percept_processing/data/model_classification.proto, blickfeld/percept_processing/data/objects.proto, blickfeld/percept_processing/data/state.proto, blickfeld/percept_processing/data/states.proto, blickfeld/percept_processing/data/vehicle.proto, blickfeld/percept_processing/data/volume_map.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    Dict,
    List,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import numpy as np

from ...base import (
    config as __base_config__,
    geometry as __base_geometry__,
)
from ...core_processing import data as __core_processing_data__
from ...percept_pipeline import config as __percept_pipeline_config__


@dataclass(eq=False, repr=False)
class Flag(betterproto.Message):
    """Flag which contains state combined with a change detection"""

    state: bool = betterproto.bool_field(1)
    """Current state"""

    since_timestamp: int = betterproto.uint64_field(2)
    """Timestamp since when the state did not change"""

    since_duration: int = betterproto.uint64_field(3)
    """Duration of current state"""


@dataclass(eq=False, repr=False)
class ModelClassification(betterproto.Message):
    """
    Classification result of machine learning object classification model.
    """

    predicted_class: "__percept_pipeline_config__.ObjectClass" = (
        betterproto.message_field(1)
    )
    """Predicted class label."""

    confidence_score: float = betterproto.float_field(2)
    """
    Value in the range of [0, 1] which indicates how confident the model is
    about the predicted label.
    """


@dataclass(eq=False, repr=False)
class DetectedObject(betterproto.Message):
    """
    A message representing exactly one object that is detected in the scene.
    """

    pose: "__base_geometry__.Pose" = betterproto.message_field(1)
    """The position and orientation of the object."""

    shape: "__base_geometry__.Shape" = betterproto.message_field(2)
    """The shape representing the object (bounding box)."""

    properties: "DetectedObjectProperties" = betterproto.message_field(3)
    """Properties of the object."""

    point_cloud: "__core_processing_data__.Frame" = betterproto.message_field(4)
    """The point cloud of the object."""

    timestamp: int = betterproto.uint64_field(5)
    """The timestamps when the object is detected for the first time"""

    lifetime: int = betterproto.uint64_field(11)
    """Lifetime of the object"""

    path: List["__base_geometry__.Pose"] = betterproto.message_field(6)
    """The tracked path of the object."""

    visible_to_sensors: List[str] = betterproto.string_field(7)
    """List of sensors which currently detect the object"""

    intruding: "Flag" = betterproto.message_field(8)
    """
    Active when object is currently detected as intruder in a security zone
    """

    intruder: "Flag" = betterproto.message_field(9)
    """Set when object intruded once in a security zone"""

    velocity: "__base_geometry__.Vector3" = betterproto.message_field(13)
    """Track object velocity"""

    geolocation: "__base_config__.Geolocation" = betterproto.message_field(14)
    """
    The geolocation of the object based on the map and the object position.
    Only set when the geolocation is configured in the data source.
    """


@dataclass(eq=False, repr=False)
class DetectedObjectProperties(betterproto.Message):
    """Properties of an object."""

    num_points: int = betterproto.uint32_field(2)
    """/ Number of measurements that belong to the object point cloud."""

    average_intensity: int = betterproto.uint32_field(3)
    """/ Average intensity of the measurements from the object point cloud."""

    bounding_box_2_d_surface: float = betterproto.float_field(4)
    """
    Surface area of the largest xz / yz bounding box plane. This is used for
    the simple object size classification.
    """

    confidence: float = betterproto.float_field(5)
    """
    Confidence value   Value in the range of [0, 1] which indicates how
    confident the tracking algorithm is about the detected object.   NOTE: This
    is preliminary implementation and might change its behavior significantly
    in upcoming releases.
    """

    in_background: bool = betterproto.bool_field(6)
    """
    Set when bounding box of object is mostly located within the background /
    reference frame. This is usually an indication for ground or vegetation
    detections.
    """

    track_length: float = betterproto.float_field(7)
    """The estimated track length of an object."""

    size: "__percept_pipeline_config__.ObjectSize" = betterproto.enum_field(
        1, group="classification_type"
    )
    """Object size classification using rule-based methods."""

    model_classification: "ModelClassification" = betterproto.message_field(
        8, group="classification_type"
    )
    """Object classification based on machine learning."""


@dataclass(eq=False, repr=False)
class Disqualification(betterproto.Message):
    """
    A message that contains different types of disqualifications for a sensor.
    """

    fog: bool = betterproto.bool_field(1)
    """Fog was detected in the point cloud."""


@dataclass(eq=False, repr=False)
class Vehicle(betterproto.Message):
    """
    A message representing exactly one vehicle that is detected in the scene.
    """

    timestamp: int = betterproto.uint64_field(1)
    """Timestamp of creation"""

    position: "__base_geometry__.Vector2" = betterproto.message_field(2)
    """The position of the vehicle"""

    dimensions: "__base_geometry__.Vector3" = betterproto.message_field(3)
    """Dimensions of the vehicle"""

    velocity: float = betterproto.float_field(4)
    """Velocity of the vehicle (in y-direction)"""

    point_cloud: "__core_processing_data__.Frame" = betterproto.message_field(5)
    """Point cloud"""

    model_classification: "ModelClassification" = betterproto.message_field(6)
    """Vehicle class"""


@dataclass(eq=False, repr=False)
class State(betterproto.Message):
    """
    A state transmits high-level information about the scene or setup. The
    information it contains depends on the configuration. States will get
    calculated on every frame of measurement.
    """

    generator_name: str = betterproto.string_field(8)
    """The name of the entity generating this state"""

    security: "StateSecurity" = betterproto.message_field(5, group="state_type")
    """The security state contains intrusion information of a zone"""

    volume: "StateVolume" = betterproto.message_field(7, group="state_type")
    """Current volume measured in the selected zone"""

    object_based_security: "StateObjectBasedSecurity" = betterproto.message_field(
        9, group="state_type"
    )
    """
    The object based security state contains intrusion information of a zone
    and the objects that trigger the intrusion
    """

    occupancy: "StateOccupancy" = betterproto.message_field(10, group="state_type")
    """The state contains occupation information of a zone"""

    traversal_security: "StateTraversalSecurity" = betterproto.message_field(
        11, group="state_type"
    )
    """The traversal state"""

    traffic_lane: "StateTrafficLane" = betterproto.message_field(12, group="state_type")
    """The state of a traffic lane zone"""


@dataclass(eq=False, repr=False)
class StateObjectBasedSecurity(betterproto.Message):
    """A message containing an alarm state of a zone."""

    alarm: bool = betterproto.bool_field(1)
    """If there is an alarm or not in the zone."""

    objects: Dict[str, "DetectedObject"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    Map of objects where the key is the UUID of the object that triggered the
    alarm.
    """

    intrusion: "Flag" = betterproto.message_field(3)
    """Active intrusion. Can be used as pre-alarm."""

    intrusion_confidence: float = betterproto.float_field(4)
    """
    Confidence that it is a real intrusion.   Calculates a moving average of
    the intrusion state. The moving average window size can be configured with
    the minimum intruding duration parameter.
    """


@dataclass(eq=False, repr=False)
class StateSecurity(betterproto.Message):
    """A message containing an alarm state of a zone."""

    alarm: bool = betterproto.bool_field(1)
    """If there is an alarm or not in the zone."""

    num_detected_points: int = betterproto.uint32_field(2)
    """The current number of lidar points detected in the zone."""

    point_cloud: "__core_processing_data__.Frame" = betterproto.message_field(3)
    """Foreground point cloud inside zone which triggered the alarm."""

    intrusion: "Flag" = betterproto.message_field(4)
    """Active intrusion. Can be used as pre-alarm."""

    intrusion_confidence: float = betterproto.float_field(5)
    """
    Confidence that it is a real intrusion.   Calculates a moving average of
    the intrusion state. The moving average window size can be configured with
    the minimum intruding duration parameter.
    """


@dataclass(eq=False, repr=False)
class StateVolume(betterproto.Message):
    """
    A message containing the volume state of a zone. This is based on one
    volume zone.
    """

    volume: float = betterproto.float_field(1)
    """The current numerical volume of the zone."""

    coverage: float = betterproto.float_field(5)
    """The current zone volume surface coverage"""


@dataclass(eq=False, repr=False)
class StateOccupancy(betterproto.Message):
    """A message containing the occupancy state of a zone."""

    occupied: bool = betterproto.bool_field(1)
    """True if the zone is occupied."""

    num_detected_points: int = betterproto.uint32_field(2)
    """The current number of lidar points detected in the zone."""

    baseline: float = betterproto.float_field(3)
    """Current baseline."""


@dataclass(eq=False, repr=False)
class StateTraversalSecurity(betterproto.Message):
    """A message containing the traversal state of a zone."""

    alarm: bool = betterproto.bool_field(1)
    """The traversal alarm"""

    objects: Dict[str, "DetectedObject"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """The objects which triggered the inbound alarm"""

    intrusion: "Flag" = betterproto.message_field(3)
    """Active traversal"""


@dataclass(eq=False, repr=False)
class StateTrafficLane(betterproto.Message):
    """A message containing the state of a traffic lane zone."""

    vehicles: Dict[str, "Vehicle"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """All vehicles currently inside the zone"""


@dataclass(eq=False, repr=False)
class VolumeMap(betterproto.Message):
    """A message containing the volume map of one volume zone."""

    timestamp: int = betterproto.uint64_field(1)
    """Timestamp of the lidar frame in which the volume map was calculated"""

    generator_uuid: str = betterproto.string_field(2)
    """The uuid of the entity generating this state"""

    generator_name: str = betterproto.string_field(3)
    """The name of the entity generating this state"""

    shape: "__base_geometry__.Shape" = betterproto.message_field(4)
    """
    The shape of the volume map, defining the pose and the dimension of the
    volume map in world frame. The shape is similar to the shape of the input
    zone with the following difference: - shape.pose.position:
    (x=zone.pose.position.x, y=zone.pose.position.y, z=0) -
    shape.pose.rotation: (roll=0, pitch=0, yaw=zone.pose.rotation.yaw) -
    shape.box.dimension: (x=zone.box.dimension.x, y=zone.box.dimension.y,z=0)
    """

    tiles: "VolumeMapTiles" = betterproto.message_field(5)
    """
    Volume map 'tiles' as defined in Tiles, providing information for each tile
    in the volume map.
    """

    tile_size: float = betterproto.float_field(6)
    """
    The tile size of the volume map (dimension of one tile in x and y
    direction)
    """

    volume: float = betterproto.float_field(7)
    """The current volume of the zone"""


@dataclass(eq=False, repr=False)
class VolumeMapTiles(betterproto.Message):
    """Tiles of the volume map"""

    length: int = betterproto.uint32_field(1)
    """Number of tiles"""

    index: np.ndarray = betterproto.bytes_field(2, numpy_dtype="(2,)<i2")
    """
    Volume Map Tile Index X (Int16) and Y (Int16) (can be negative and
    positive) The indices of Volume Tiles are expressed w.r.t the Volume Map
    coordinate system. The tile with index (0,0) will contain the points from
    [0, tile_size), [0, tile_size). That means that (0,0,0) of the Volume Map
    frame does not coincide with the center of the tile with index (0,0) but
    with the crossing of the 4 tiles [(0,0), (0, -1,), (-1,-1), (-1,0)]. Type:
    (Int16, Int16)
    """

    height: np.ndarray = betterproto.bytes_field(3, numpy_dtype="<f4")
    """Height per tile, ordered ascending Type: Float32"""

    std_deviation: np.ndarray = betterproto.bytes_field(4, numpy_dtype="<f4")
    """Standard Deviation per tile Type: Float32"""

    number_points: np.ndarray = betterproto.bytes_field(5, numpy_dtype="<u2")
    """Number Points per tile Type: UInt16"""


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """
    Event which is generated based on continuous zone state.   In contrast to a
    state, an event is emitted only for changes. Thus, an event is often
    directly actionable.
    """

    timestamp: int = betterproto.uint64_field(1)
    """Unix timestamp of event"""

    security_intrusion_started: "EventSecurityIntrusionStarted" = (
        betterproto.message_field(2, group="type")
    )
    """Security: Intrusion started"""

    security_intrusion_ended: "EventSecurityIntrusionEnded" = betterproto.message_field(
        3, group="type"
    )
    """Security: Intrusion ended"""

    security_zone_intrusion_started: "EventSecurityZoneIntrusionStarted" = (
        betterproto.message_field(4, group="type")
    )
    """Security: Zone intrusion started"""

    security_zone_intrusion_ended: "EventSecurityZoneIntrusionEnded" = (
        betterproto.message_field(5, group="type")
    )
    """Security: Zone intrusion ended"""

    disqualification_fog_detected: "EventDisqualificationFog" = (
        betterproto.message_field(6, group="type")
    )
    """Disqualification: Fog detected"""

    disqualification_fog_stopped: "EventDisqualificationFog" = (
        betterproto.message_field(7, group="type")
    )
    """Disqualification: Fog not detected anymore"""

    traffic_vehicle_detected: "EventTrafficVehicleDetected" = betterproto.message_field(
        8, group="type"
    )
    """Traffic: Vehicle detected upon leaving a lane zone"""


@dataclass(eq=False, repr=False)
class EventSecurity(betterproto.Message):
    """Container for security-related events"""

    pass


@dataclass(eq=False, repr=False)
class EventSecurityIntrusionStarted(betterproto.Message):
    """An intrusion started in one or multiple security zones."""

    pass


@dataclass(eq=False, repr=False)
class EventSecurityIntrusionEnded(betterproto.Message):
    """An intrusion ended."""

    started_event_id: str = betterproto.string_field(1)
    """ID of the IntrusionStarted event."""

    duration: float = betterproto.float_field(2)
    """Duration of the intrusion event."""


@dataclass(eq=False, repr=False)
class EventSecurityZoneIntrusionStarted(betterproto.Message):
    """An intrusion started in a security zone."""

    zone_uuid: str = betterproto.string_field(1)
    """UUID of the security zone."""

    state: "State" = betterproto.message_field(2)
    """Snapshot of the zone state."""


@dataclass(eq=False, repr=False)
class EventSecurityZoneIntrusionEnded(betterproto.Message):
    """An intrusion of a security zone ended."""

    zone_uuid: str = betterproto.string_field(1)
    """UUID of the security zone."""

    started_event_id: str = betterproto.string_field(2)
    """ID of the ZoneIntrusionStarted event."""

    duration: float = betterproto.float_field(3)
    """Duration of the intrusion event."""


@dataclass(eq=False, repr=False)
class EventTraffic(betterproto.Message):
    """Container for traffic-related events"""

    pass


@dataclass(eq=False, repr=False)
class EventTrafficVehicleDetected(betterproto.Message):
    """A vehicle was detected upon leaving a zone"""

    zone_uuid: str = betterproto.string_field(1)
    """UUID of the traffic lane zone."""

    vehicle: "Vehicle" = betterproto.message_field(2)
    """The detected vehicle"""

    id: str = betterproto.string_field(3)
    """The id of the detected vehicle"""


@dataclass(eq=False, repr=False)
class EventDisqualification(betterproto.Message):
    """Container for disqualification-related events"""

    pass


@dataclass(eq=False, repr=False)
class EventDisqualificationFog(betterproto.Message):
    """Fog was detected in the point cloud."""

    pass


@dataclass(eq=False, repr=False)
class Objects(betterproto.Message):
    """
    A message that contains a map of objects. An object contains the
    information regarding the detected object by the system.
    """

    timestamp: int = betterproto.uint64_field(1)
    """The timestamp of the measurement(s) used to compute the objects."""

    objects: Dict[str, "DetectedObject"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    Map of objects where the key is the UUID of the object. These objects
    should contain the point cloud of each object.
    """


@dataclass(eq=False, repr=False)
class States(betterproto.Message):
    """
    A message that contains a map of states. A state is an information
    generated by the system based on the select zone type or device. For
    example, a state can contain information about the volume or the intrusion
    in the zone.
    """

    timestamp: int = betterproto.uint64_field(1)
    """The timestamp of the measurement(s) used to compute the states."""

    states: Dict[str, "State"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Map of states where the key is the generator UUID of the state."""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/system/data/firmware.proto, blickfeld/system/data/health.proto, blickfeld/system/data/network_status.proto, blickfeld/system/data/scan_pattern.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import (
    config as __base_config__,
    data as __base_data__,
)
from .. import config as _config__


@dataclass(eq=False, repr=False)
class Firmware(betterproto.Message):
    """
    The Firmware data message contains all relevant information about the
    current firmware, uploaded firmware and the status of an firmware
    installation.
    """

    pass


@dataclass(eq=False, repr=False)
class FirmwareInfo(betterproto.Message):
    """
    contains information about rauc bundle parsed from the associated manifest
    file
    """

    label: str = betterproto.string_field(1)
    """Human-readable label / version of the firmware"""

    version: "__base_data__.Version" = betterproto.message_field(2)
    """Version"""

    allowed_downgrade_version: "__base_data__.Version" = betterproto.message_field(3)
    """
    Lowest allowed version which can be downgraded to if this firmware is
    installed
    """


@dataclass(eq=False, repr=False)
class FirmwareStatus(betterproto.Message):
    """contains information about rauc bridge status"""

    idle: "FirmwareStatusIdle" = betterproto.message_field(1, group="status")
    """no bundle is available and no installation is happenning"""

    ready_to_install: "FirmwareStatusReadyToInstall" = betterproto.message_field(
        2, group="status"
    )
    """bundle is verified and available for installation"""

    installation: "FirmwareStatusInstallation" = betterproto.message_field(
        3, group="status"
    )
    """the installation process is ongoing"""

    failure: "FirmwareStatusFailure" = betterproto.message_field(4, group="status")
    """installation failed"""

    reboot: "FirmwareStatusReboot" = betterproto.message_field(5, group="status")
    """reboot has been triggered after successfull installation"""

    installed_firmware_info: "FirmwareInfo" = betterproto.message_field(6)
    """Currently installed firmware"""


@dataclass(eq=False, repr=False)
class FirmwareStatusIdle(betterproto.Message):
    """
    initial state when no bundle is available and no installation is happenning
    """

    pass


@dataclass(eq=False, repr=False)
class FirmwareStatusReadyToInstall(betterproto.Message):
    """state when bundle is verified and available for installation"""

    uploaded_firmware_info: "FirmwareInfo" = betterproto.message_field(1)
    """Firmware info of bundle which has been uploaded"""


@dataclass(eq=False, repr=False)
class FirmwareStatusInstallation(betterproto.Message):
    """state when the installation process is ongoing"""

    progress: "FirmwareStatusInstallationProgress" = betterproto.message_field(1)
    """Current installation progress"""


@dataclass(eq=False, repr=False)
class FirmwareStatusInstallationProgress(betterproto.Message):
    """
    Contains information about current state of installation procedure inside
    the tree of progress
    """

    percent: int = betterproto.uint32_field(1)
    """progress of installation in percents (0%-100%)"""

    message: str = betterproto.string_field(2)
    """log message describing the current installation step"""


@dataclass(eq=False, repr=False)
class FirmwareStatusFailure(betterproto.Message):
    """state when installation failed"""

    reason: str = betterproto.string_field(1)
    """Human-readable reason for installation failure"""


@dataclass(eq=False, repr=False)
class FirmwareStatusReboot(betterproto.Message):
    """
    state when reboot of the device has been triggered after successfull
    installation
    """

    pass


@dataclass(eq=False, repr=False)
class NetworkStatus(betterproto.Message):
    """
    Carries information about active connection: current statistics and network
    configuration
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(3)
    """High-level state of the time synchronization"""

    state_reason: str = betterproto.string_field(4)
    """Reason for given state. Is not set if state is STATE_OK."""

    active_config: "_config__.Network" = betterproto.message_field(1)
    """Brings configuration information about currently active connection"""

    link_speed: int = betterproto.uint32_field(5)
    """Link speed of network interface"""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Message representing health"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is STATE_OK."""

    time_synchronization: "HealthTimeSynchronization" = betterproto.message_field(3)
    """Time synchronization status"""

    network: "NetworkStatus" = betterproto.message_field(4)
    """Network status"""


@dataclass(eq=False, repr=False)
class HealthTimeSynchronization(betterproto.Message):
    """Status message for time synchronization"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of the time synchronization"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is STATE_OK."""

    current_system_timestamp: int = betterproto.uint64_field(3)
    """Current timestamp"""

    current_time_zone: str = betterproto.string_field(5)
    """Current time zone"""

    ntp: "HealthTimeSynchronizationNtp" = betterproto.message_field(4, group="type")
    """Status for NTP time synchronization"""


@dataclass(eq=False, repr=False)
class HealthTimeSynchronizationNtp(betterproto.Message):
    """Status message representing the NTP time synchronization status"""

    synchronized: bool = betterproto.bool_field(1)
    """Flag indicating that the system time is synchronized with NTP"""

    server: str = betterproto.string_field(2)
    """The server NTP time server which is used"""

    offset: float = betterproto.float_field(3)
    """Time offset between Qb2 system time and time server"""

    delay: float = betterproto.float_field(4)
    """Time delay from NTP request until server response is received"""

    jitter: float = betterproto.float_field(5)
    """Jitter between multiple time comparisons"""

    precision: float = betterproto.float_field(6)
    """Precision of the NTP time synchronization"""


@dataclass(eq=False, repr=False)
class ScanPatternLimits(betterproto.Message):
    """
    This section contains limits for configurable parameters of a scan pattern
    """

    horizontal: "ScanPatternLimitsHorizontal" = betterproto.message_field(1)
    vertical: "ScanPatternLimitsVertical" = betterproto.message_field(2)
    frame_rate: "ScanPatternLimitsFrameRate" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ScanPatternLimitsHorizontal(betterproto.Message):
    """
    This section defines the value limits for the horizontal configuration
    """

    field_of_view: "__base_config__.RangeFloat" = betterproto.message_field(1)
    """The field of view limits"""


@dataclass(eq=False, repr=False)
class ScanPatternLimitsVertical(betterproto.Message):
    """This section defines the value limits for the vertical configuration"""

    field_of_view: "__base_config__.RangeFloat" = betterproto.message_field(1)
    """The field of view limits"""

    scanlines_up: "__base_config__.RangeUint32" = betterproto.message_field(2)
    """The up-ramping scanline limits"""

    scanlines_down: "__base_config__.RangeUint32" = betterproto.message_field(3)
    """The down-ramping scanline limits"""

    foveation: "ScanPatternLimitsVerticalFoveation" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ScanPatternLimitsVerticalFoveation(betterproto.Message):
    """This section defines the limits for the foveation configuration"""

    fraction: "__base_config__.RangeFloat" = betterproto.message_field(1)
    """The foveation fraction limit"""

    density_factor: "__base_config__.RangeFloat" = betterproto.message_field(2)
    """The density factor limits"""

    scanlines: "__base_config__.RangeUint32" = betterproto.message_field(3)
    """The foveated scanlines limits"""


@dataclass(eq=False, repr=False)
class ScanPatternLimitsFrameRate(betterproto.Message):
    """
    This section defines the value limits for the frame rate configuration
    """

    target: "__base_config__.RangeDouble" = betterproto.message_field(1)
    """The target frame rate limit"""





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/config/data/watch_event.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class WatchEventType(betterproto.Enum):
    """Type of watch event"""

    TYPE_UNSPECIFIED = 0
    """Unspecified type"""

    TYPE_CREATED = 1
    """A value for a key or map entry has been stored the first time"""

    TYPE_UPDATED = 2
    """A value for a key or map entry has been updated"""

    TYPE_DELETED = 3
    """A value for a key or map entry has been deleted"""


@dataclass(eq=False, repr=False)
class WatchEvent(betterproto.Message):
    """Watch event which is emitted on a config change"""

    pass





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/core_processing/data/acceleration.proto, blickfeld/core_processing/data/acceleration_buffer.proto, blickfeld/core_processing/data/frame.proto, blickfeld/core_processing/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import numpy as np

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Frame(betterproto.Message):
    """This section describes the contents of a point cloud frame."""

    id: int = betterproto.uint64_field(1)
    """Incremental frame ID since startup of the device."""

    timestamp: int = betterproto.uint64_field(2)
    """
    Timestamp of start of frame. Timestamp is synchronized if Time
    Synchronization is configured.
    """

    binary: "FrameBinary" = betterproto.message_field(3)
    """Binary buffer with point cloud vectors"""


@dataclass(eq=False, repr=False)
class FrameBinary(betterproto.Message):
    """
    The fields of a point cloud frame are encoded as binary arrays. Binary
    types instead of native Protobuf types are used to improve the performance.
    NOTE: All byte arrays are encoded in Little Endian.
    """

    length: int = betterproto.uint32_field(1)
    """Number of entries in arrays below."""

    cartesian: np.ndarray = betterproto.bytes_field(2, numpy_dtype="(3,)<f4")
    """
    Cartesian coordinates. 3-dimensional array in row-major format with [x, y,
    z] tuples. Type: Float32
    """

    photon_count: np.ndarray = betterproto.bytes_field(3, numpy_dtype="<u2")
    """Estimated number of photons. 1-dimensional array. Type: UInt16"""

    direction_id: np.ndarray = betterproto.bytes_field(5, numpy_dtype="<u4")
    """
    Unique direction identifiers within a frame and scan pattern. 1-dimensional
    array. Type: UInt32
    """

    timestamp: np.ndarray = betterproto.bytes_field(6, numpy_dtype="<u8")
    """Unix timestamp of points. Type: UInt64"""

    metadata: np.ndarray = betterproto.bytes_field(100, numpy_dtype="<u2")
    """
    Arbitrary metadata for annotation of the binary data fields. 1-dimensional
    array. Type: UInt16
    """


@dataclass(eq=False, repr=False)
class Acceleration(betterproto.Message):
    """A measurement of the Accelerometer"""

    x: float = betterproto.float_field(1)
    """X component of acceleration vector"""

    y: float = betterproto.float_field(2)
    """Y component of acceleration vector"""

    z: float = betterproto.float_field(3)
    """Z component of acceleration vector"""


@dataclass(eq=False, repr=False)
class AccelerationBuffer(betterproto.Message):
    """Buffer for multiple Acceleration samples"""

    timestamp: int = betterproto.uint64_field(1)
    """Timestamp of the first sample in the buffer."""

    sampling_period: int = betterproto.uint64_field(2)
    """Sampling period of the acceleration samples"""

    cartesian: bytes = betterproto.bytes_field(3)
    """
    Cartesian 3-dimensional array in row-major format with [x, y, z] tuples.
    Type: Float32
    """


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Message representing health"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is STATE_OK."""

    channel_status: "HealthChannelStatus" = betterproto.message_field(3)
    """Status of internal channels"""

    server_status: "HealthServerStatus" = betterproto.message_field(4)
    """Status of gRPC server"""

    error_flags: "HealthErrorFlags" = betterproto.message_field(5)
    """Error flags which give insight in fine-grained errors"""


@dataclass(eq=False, repr=False)
class HealthChannelStatus(betterproto.Message):
    """Status for typed channels producing raw-data"""

    point_cloud: "HealthChannelStatusPointCloud" = betterproto.message_field(1)
    """Point Cloud status"""

    acceleration: "HealthChannelStatusAcceleration" = betterproto.message_field(2)
    """Acceleration status"""

    adc: "HealthChannelStatusAdc" = betterproto.message_field(3)
    """Adc status"""

    vertical_beam_deflection_control: "HealthChannelStatusBeamDeflectionControl" = (
        betterproto.message_field(4)
    )
    """Vertical Beam Deflection Control status"""

    horizontal_beam_deflection_control: "HealthChannelStatusBeamDeflectionControl" = (
        betterproto.message_field(5)
    )
    """Horizontal Beam Deflection Control status"""

    magnetic_field: "HealthChannelStatusMagneticField" = betterproto.message_field(6)
    """Magnetic Field status"""


@dataclass(eq=False, repr=False)
class HealthChannelStatusPointCloud(betterproto.Message):
    """Status of Point Cloud channel"""

    point_rate: float = betterproto.float_field(1)
    """Processed points per second"""

    scanline_rate: float = betterproto.float_field(2)
    """Scanlines per second"""

    frame_rate: float = betterproto.float_field(3)
    """Frames per second"""

    data_rate: float = betterproto.float_field(4)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusAcceleration(betterproto.Message):
    """Status of Acceleration channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Processed buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusAdc(betterproto.Message):
    """Status of Adc channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Processed buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusBeamDeflectionControl(betterproto.Message):
    """Status of BeamDeflectionControl channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Produced buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthChannelStatusMagneticField(betterproto.Message):
    """Status of Magnetic-Field channel"""

    buffer_rate: float = betterproto.float_field(1)
    """Produced buffers per second"""

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatus(betterproto.Message):
    """Status for Server / Client activities"""

    point_cloud: "HealthServerStatusPointCloud" = betterproto.message_field(1)
    """Point Cloud service"""

    acceleration: "HealthServerStatusAcceleration" = betterproto.message_field(2)
    """Acceleration service"""

    adc: "HealthServerStatusAdc" = betterproto.message_field(3)
    """Adc service"""

    beam_deflection_control: "HealthServerStatusBeamDeflectionControl" = (
        betterproto.message_field(4)
    )
    """Beam Deflection Control service"""

    health: "HealthServerStatusHealth" = betterproto.message_field(5)
    """Health service"""

    magnetic_field: "HealthServerStatusMagneticField" = betterproto.message_field(6)
    """Magnetic Field service"""


@dataclass(eq=False, repr=False)
class HealthServerStatusPointCloud(betterproto.Message):
    """Status for Point Cloud service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections   [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.   [NOTE] This field is deprecated, use the field
    `clients` instead.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """

    clients: Dict[str, "HealthServerStatusPointCloudClient"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """List of currently connected client IPs and their information"""


@dataclass(eq=False, repr=False)
class HealthServerStatusPointCloudClient(betterproto.Message):
    """Connected client information"""

    number_of_streams: int = betterproto.uint32_field(1)
    """Number of open streams belonging to the client."""

    lost_frames: int = betterproto.uint64_field(2)
    """
    Number of frames which have been lost due to slow reading from the client
    side.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusAcceleration(betterproto.Message):
    """Status for Acceleration service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections   [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusAdc(betterproto.Message):
    """Status for Adc service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections   [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusBeamDeflectionControl(betterproto.Message):
    """Status for BeamDeflectionControl service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections   [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusHealth(betterproto.Message):
    """Status for Health service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections   [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthServerStatusMagneticField(betterproto.Message):
    """Status for Magnetic Field service"""

    number_of_clients: int = betterproto.uint32_field(1)
    """
    Number of currently open client connections   [NOTE] This is a snapshot and
    does not distinguish between long running streams or single request /
    response methods.
    """

    data_rate: float = betterproto.float_field(2)
    """
    Average data rate   [NOTE] This value is averaged over 10 seconds, peak or
    current values might be different.
    """


@dataclass(eq=False, repr=False)
class HealthErrorFlags(betterproto.Message):
    """Error flags of internal data handling mechanisms"""

    packet_counter_mismatch_packager: bool = betterproto.bool_field(1)
    """TLP counter mismatch for Packager DMA"""

    packet_counter_mismatch_debug: bool = betterproto.bool_field(2)
    """TLP counter mismatch for Debug DMA"""

    exhausted_scanline_buffers: bool = betterproto.bool_field(3)
    """Dropping data because scanline buffers are exhausted"""

    exhausted_acceleration_buffers: bool = betterproto.bool_field(4)
    """Dropping data because acceleration buffers are exhausted"""

    exhausted_adc_buffers: bool = betterproto.bool_field(5)
    """Dropping data because adc buffers are exhausted"""

    exhausted_vertical_beam_deflection_control_buffers: bool = betterproto.bool_field(6)
    """
    Dropping data because vertical beam deflection control buffers are
    exhausted
    """

    exhausted_horizontal_beam_deflection_control_buffers: bool = betterproto.bool_field(
        7
    )
    """
    Dropping data because horizontal beam deflection control buffers are
    exhausted
    """

    exhausted_magnetic_field_buffers: bool = betterproto.bool_field(8)
    """Dropping data because magnetic field buffers are exhausted"""

    client_loosing_frames: bool = betterproto.bool_field(9)
    """
    One or multiple clients are loosing frames because they are reading too
    slow
    """

    point_cloud_stream_not_active: bool = betterproto.bool_field(10)
    """The point cloud stream is not active"""

    point_rate_too_low: bool = betterproto.bool_field(11)
    """
    The number of points in the point cloud is too low. One of the reasons is
    that the device is covered. The point rate limit that is used to trigger
    this flag can be configured with Health.SetConfig API method.
    """

    point_cloud_alignment_changed: bool = betterproto.bool_field(12)
    """
    The point cloud alignment deviates from the reference alignment. This might
    have an impact on the point cloud processing as e.g. the ground alignment
    got incorrect. If this is expected, the alignment can be reconfigured with
    the Health.SetConfig API method.
    """





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_pipeline/data/coordinate_system.proto, blickfeld/percept_pipeline/data/health.proto, blickfeld/percept_pipeline/data/point_cloud_type.proto, blickfeld/percept_pipeline/data/point_type.proto, blickfeld/percept_pipeline/data/state.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


class CoordinateSystem(betterproto.Enum):
    """Specifies the coordinates system the data is expressed in"""

    COORDINATE_SYSTEM_UNSPECIFIED = 0
    """
    Coordinate system not specified. Not accepted neither within a request nor
    as response
    """

    COORDINATE_SYSTEM_LOCAL = 1
    """
    The data is expressed in the coordiante system of the data-source/device
    """

    COORDINATE_SYSTEM_TRANSFORMED = 2
    """The data is expressed in a map coordinate system"""


class PointCloudType(betterproto.Enum):
    """The type of the point cloud"""

    POINT_CLOUD_TYPE_UNSPECIFIED = 0
    """The zero value that is not accepted"""

    POINT_CLOUD_TYPE_FULL = 1
    """
    Full point cloud (combined point cloud of all devices or unfiltered point
    cloud coming from one device)
    """

    POINT_CLOUD_TYPE_FOREGROUND = 2
    """Point cloud containing only foreground/points from moving objects"""

    POINT_CLOUD_TYPE_FULL_OPTIMIZED = 3
    """
    Same as 'POINT_CLOUD_TYPE_FULL' but with reduced point cloud density to
    reduce the size of the point cloud
    """

    POINT_CLOUD_TYPE_FILTERED = 4
    """
    Same as 'POINT_CLOUD_TYPE_FULL' but with all points excluded that are in
    exclusion zones
    """

    POINT_CLOUD_TYPE_FILTERED_FOREGROUND = 5
    """
    Same as 'POINT_CLOUD_TYPE_FOREGROUND' but with all points excluded that are
    in exclusion zones
    """

    POINT_CLOUD_TYPE_BACKGROUND = 6
    """
    Point cloud containing the points representing the background model/point
    cloud
    """

    POINT_CLOUD_TYPE_ANNOTATED = 7
    """
    Point cloud containing a collection of points with annotation per point
    """


class State(betterproto.Enum):
    """The states of pipeline"""

    STATE_UNSPECIFIED = 0
    """State field is not set"""

    STATE_STARTING = 1
    """Pipeline is getting started currently."""

    STATE_STABLE = 2
    """
    Pipeline is stable input and output of the pipeline are achieving the same
    frequency. Only very few frames are allowed to be dropped.
    """

    STATE_UNSTABLE = 3
    """
    Pipeline is unstable if the pipeline cannot process all frames of the data
    source and thus has to drop frames to keep up.
    """

    STATE_BROKEN = 4
    """
    Pipeline is broken if the pipeline does not generate the expected outputs,
    i.e. frames or state list at all. This can happen if the devices in the
    'data source' cannot be reached.
    """

    STATE_RESTARTING = 5
    """
    Pipeline is restarting if there is a larger configuration change, i.e. data
    source changed.
    """

    STATE_IDLE = 6
    """No processing is happening in the pipeline."""


class PointType(betterproto.Enum):
    """The type of the point"""

    POINT_TYPE_UNSPECIFIED = 0
    """The point type is unspecified (default)."""

    POINT_TYPE_FOREGROUND = 1
    """The point is part of the foreground point cloud."""

    POINT_TYPE_BACKGROUND = 2
    """The point is part of the background point cloud."""

    POINT_TYPE_FILTERED = 4
    """
    The point is filtered, e.g. by applying an exclusion zone or the radius
    outlier filter.
    """

    POINT_TYPE_SHADOW = 8
    """The point is part of a shadow."""

    POINT_TYPE_OBJECT = 256
    """The point is part of a detected object point cloud."""

    POINT_TYPE_OBJECT_INTRUDING = 512
    """
    The point is part of a detected object point cloud which is currently
    intruding a object based security zone.
    """

    POINT_TYPE_OBJECT_INTRUDER = 1024
    """
    The point is part of a detected object point cloud which was previously
    intruding a object based security zone.
    """


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    A health message that contains information about the pipeline status and
    the module itself.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module."""

    state_reason: str = betterproto.string_field(2)
    """Reason for the given state. It is not set if state is STATE_OK."""

    data_source: "HealthDataSource" = betterproto.message_field(4)
    """Data source related health information."""


@dataclass(eq=False, repr=False)
class HealthDataSource(betterproto.Message):
    """Data source related health."""

    device: Dict[str, "HealthDataSourceDevice"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Map of devices. The key is the serial number of the device."""


@dataclass(eq=False, repr=False)
class HealthDataSourceDevice(betterproto.Message):
    """Configured device."""

    error_flags: "HealthDataSourceDeviceErrorFlags" = betterproto.message_field(1)
    """Error flags"""


@dataclass(eq=False, repr=False)
class HealthDataSourceDeviceErrorFlags(betterproto.Message):
    """Error flags"""

    connect_failed: bool = betterproto.bool_field(1)
    """
    The connection to the device has failed. This is most probably caused by
    power supply or network connectivity issues.
    """

    start_failed: bool = betterproto.bool_field(2)
    """
    The start-up of the device failed. This indicates that the device can not
    operate properly. Please review the diagnostics information of the device.
    """

    point_cloud_fetch_failed: bool = betterproto.bool_field(3)
    """
    The point cloud fetch failed. This indicates that the device can not
    operate properly or network bandwidth / stability issues.
    """

    environmental_effects: bool = betterproto.bool_field(4)
    """
    There are some environmental influences, e.g. fog, rain or snow in the
    scene.
    """





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/beam_deflection_control/config/module.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class Module(betterproto.Enum):
    """
    This enum defines the different modules that can run the beam deflection
    control firmware and thus can be communicated with. It needs to be passed
    as metadata when opening the gRPC channel in order to select the module.
    """

    MODULE_UNSPECIFIED = 0
    """Unused zero value"""

    MODULE_VERTICAL = 1
    """Vertical beam deflection unit"""

    MODULE_HORIZONTAL = 2
    """Horizontal beam deflection unit"""
